//----------------------------------------------------------------
// DirectGlide.cpp : Defines the entry point for the application.
// By: Aaron Hilton
// Copylefted (c) January 1999
//----------------------------------------------------------------
// Notes:
//  "TBI - " is where something needs to be implemented.
//------------------------------------------------------

#include "stdafx.h"
#include "resource.h"


DirectGlideSettings dgs;

// DirectGlideProc - Handles the regular windows messages.
BOOL CALLBACK DirectGlideProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );

// ConfirmDevice - Callback; Called during device intialization, this code
//  checks the device for some minimum set of capabilities.
HRESULT ConfirmDevice( DDCAPS*, D3DDEVICEDESC* );

// UpdateInfo - Sets the display info into the dialog box.
void UpdateInfo( HWND wnd );

// PrepareSystem - Performs preparation tasks to run DirectGlide
//					with any game, using a Non-3Dfx card.
void PrepareSystem( HWND wnd );

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	// Enumerate available D3D devices, passing a callback that allows devices
	// to be accepted/rejected based on what capabilities the app requires.
	HRESULT hr;
	if( FAILED( hr = D3DEnum_EnumerateDevices( ConfirmDevice ) ) )
	{
		MessageBox( NULL, "Failed to enumerate any 3D devices on your system.", "Error", MB_OK);
		return 0;
	}

	// Load our settings from the registry.
	dgs.Load();
	
	// Load up the accelerator hot keys.
	HACCEL hdgaccel = LoadAccelerators( hInstance, MAKEINTRESOURCE(IDR_DGACCEL ) );

	// Create the all important dialog box.
	HWND hwnd = CreateDialog( hInstance, MAKEINTRESOURCE(IDD_DIRECTGLIDE), NULL, DirectGlideProc );
	if( hwnd == 0 )
	{
		MessageBox( NULL, "Crap!  Something prevented DirectGlide Setup to open its dialog box!", "Blame Microsoft", MB_OK );
		return 0;
	}

	// Update the text fields in the dialog box.
	UpdateInfo( hwnd );

	// Get the dialog up on screen.
	ShowWindow( hwnd, SW_SHOWDEFAULT );
	UpdateWindow( hwnd );

	// Process the messages.
	MSG msg;
	while(GetMessage( &msg, NULL, 0, 0 ) )
	{
		if( !TranslateAccelerator( hwnd, hdgaccel, &msg ) )
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
	}

	return msg.wParam; 
}




//---------------------
// DirectGlideProc - Handles the messages passed to the setup dialog box.
// Jan 24, 1998
//---------------------
BOOL CALLBACK DirectGlideProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch( uMsg )
	{
	case WM_INITDIALOG:
		return TRUE;
	
	// case WM_CREATE:
		break;

	case WM_DESTROY:
		PostQuitMessage( 0 );
		return TRUE;

	case WM_COMMAND:
		{
			WORD wNotifyCode = HIWORD(wParam); // notification code 
			WORD wID = LOWORD(wParam);         // item, control, or accelerator identifier 
			HWND hwndCtl = (HWND) lParam;      // handle of control 

			switch( wID )
			{
			case IDC_SELECT:
				// Pull up the interface to select our Direct3D settings.
				D3DEnum_UserDlgSelectDriver( hwndDlg, TRUE );

				// Aquire the settings out of the Direct3D framework.
				dgs.Aquire();

				// Update the dialog box.
				UpdateInfo( hwndDlg );
				return TRUE;

			case IDC_TEST:
				MessageBox( hwndDlg, "This function is not implemented yet.", "Notice", MB_OK );
				return TRUE;

			case IDC_INSTALL_TOGAME:
				MessageBox( hwndDlg, "This function is not implemented yet.", "Notice", MB_OK );
				return TRUE;

			case IDC_INSTALL_TOSYSTEM:
				PrepareSystem( hwndDlg );
				return TRUE;

			case IDOK:
				dgs.Save();
				DestroyWindow( hwndDlg );
				return TRUE;

			case IDCANCEL:
				DestroyWindow( hwndDlg );
				return TRUE;
			}
		}
	}


	return FALSE;
}

//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Callback - Called during device intialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT ConfirmDevice( DDCAPS* pddDriverCaps,
						   D3DDEVICEDESC* pd3dDeviceDesc )
{
    return S_OK;
}


// UpdateInfo - Sets the display info into the dialog box.
void UpdateInfo( HWND wnd )
{
	SetDlgItemText( wnd, IDC_DRIVER, dgs.driverName );
	SetDlgItemText( wnd, IDC_DEVICE, dgs.deviceName );
	SetDlgItemText( wnd, IDC_MODE, dgs.modeDesc );
}


// PrepareSystem - Performs preparation tasks to run DirectGlide
//					with any game, using a Non-3Dfx card.
void PrepareSystem( HWND wnd )
{
//	char szDirectGlide[MAX_PATH], szDir[MAX_PATH], szSystemGlide[MAX_PATH]; // Processed paths/files.
	
	// TBI - Need more protection code, to prevent Newbies from nuking their real 3Dfx software.
	MessageBox( wnd, "This function is not implemented yet.", "Notice", MB_OK );
	return;

/*
	//-- Get the System Directory --
	{
		HKEY key = NULL;
		// See if we can find where the System is installed through the registry.
		if( RegOpenKeyEx( HKEY_LOCAL_MACHINE,
			"Software\\Microsoft\\Windows\\CurrentVersion", 0, KEY_READ, &key) != ERROR_SUCCESS )
		{
			// We couldn't automatically get our System path. TBI - Try asking for it.
			MessageBox( wnd, "Could not determine the windows system root directory in the registry.", "Stupid Error", MB_OK | MB_ICONEXCLAMATION );
			return;
		} else
		{
			DWORD size, type;
			// We have a key, so retrive the relevent info.
			size = sizeof(szDir);
			if( RegQueryValueEx( key, "SystemRoot", 0, &type, (LPBYTE)szDir, &size) != ERROR_SUCCESS )
			{
				MessageBox( wnd, "Failed to retrieve the path to the windows system root directory in the registry.", "Stupid Error", MB_OK | MB_ICONEXCLAMATION);
				RegCloseKey( key );
				return;
			}

			// We don't need the registry key anymore, so close it up.
			RegCloseKey( key );
		}
	}


	//-- Prepare the file paths --
	{
		// Prepare the System Glide2X.dll path.
		strcpy( szSystemGlide, szDir );
		strcat( szSystemGlide, "Glide2X.dll" );

		// Build our path to DirectGlide2X.dll
		GetModuleFileName(NULL, szDirectGlide, MAX_PATH);  // Generate our exe's path.

		// Clip the filename off the end of the path.
		char *p=szDirectGlide+strlen(szDirectGlide);
		while (p >= szDirectGlide && *p != '\\') p--;
		if (++p >= szDirectGlide) *p = 0;

		// Append our DirectGlide2X.dll filename.
		strcat( szDirectGlide, "DirectGlide2X.dll");
	}

	//-- Prepare for the Copy --
	{
		// First, determine if glide2x.dll exists in Windows\System.
		WIN32_FIND_DATA findData;
		HANDLE hFind;

		// Try finding the file, first.
		hFind = FindFirstFile( szSystemGlide, &findData);
		if( hFind != INVALID_HANDLE_VALUE )
		{
			// We found a valid file, so close up the FindFile routines.
			FindClose( hFind );

			// Ask to replace Glide2X.dll in the Windows\System directory.
			if( MessageBox( wnd, "Glide2x.dll was located in the Windows\\System directory, replace it?", "Progress Blocker", MB_OKCANCEL | MB_ICONQUESTION) == IDCANCEL)
			{
				MessageBox( wnd, "Preparation could not complete. Canceling.", "Error", MB_OK | MB_ICONEXCLAMATION );
				return;
			}
		}
	}


	//-- Do the actual Glide2X.dll copy --
	{
		// Copy the Glide2x.dll file from the DirectGlide.exe's directory,
		// and toss it in the Windows\System directory.
		CopyFile( szDirectGlide, szSystemGlide, FALSE );
	}

	// That's it!  We're outa here....
	MessageBox( wnd, "The system is now ready to run with DirectGlide.", "How Wonderful", MB_OK );
	*/
}

