// DirectGlideSettings.cpp : A class which stores the settings for the glide wrapper.
//

#include "stdafx.h"
#include "DirectGlideSettings.h"

// Load the DirectGlide settings from the registry.
//  if they are not found there,
//	 then we aquire the current driver from the D3DEnum routines.
//	"We expect D3DEnum_EnumerateDevices() to be called already"
bool DirectGlideSettings::Load()
{
	HKEY keyDG = NULL;

	// See if we have previous settings in the registry.
	if( RegCreateKey( HKEY_CURRENT_USER, "Software\\DirectGlide", &keyDG) != ERROR_SUCCESS )
		goto ReAquireSettings;

	DWORD size, type;
	// We have a key, so retrive the relevent info.
	size = sizeof(GUID);
	if( RegQueryValueEx( keyDG, "driverGUID", 0, &type, (LPBYTE)&driverGUID, &size) != ERROR_SUCCESS )
		goto ReAquireSettings;
	size = 256;
	if( RegQueryValueEx( keyDG, "driverName", 0, &type, (LPBYTE)driverName, &size) != ERROR_SUCCESS )
		goto ReAquireSettings;
	size = sizeof(GUID);
	if( RegQueryValueEx( keyDG, "deviceGUID", 0, &type, (LPBYTE)&deviceGUID, &size) != ERROR_SUCCESS )
		goto ReAquireSettings;
	size = 256;
	if( RegQueryValueEx( keyDG, "deviceName", 0, &type, (LPBYTE)deviceName, &size) != ERROR_SUCCESS )
		goto ReAquireSettings;
	size = 256;
	if( RegQueryValueEx( keyDG, "ModeDescription", 0, &type, (LPBYTE)modeDesc, &size) != ERROR_SUCCESS )
		goto ReAquireSettings;

	DWORD data;
	size = sizeof(DWORD);
	if( RegQueryValueEx( keyDG, "FullScreen", 0, &type, (LPBYTE)&data, &size) != ERROR_SUCCESS )
		goto ReAquireSettings;

	bFullScreen = (data>0) ? true : false;

	RegCloseKey( keyDG );

	return true;

ReAquireSettings:
	// Something failed, so just create our own info.

	// Make sure the registry key is closed.
	if( keyDG )
		RegCloseKey( keyDG );


	// We expect D3DEnum_EnumerateDevices() to be called already.
	D3DEnum_SelectDefaultDriver( D3DENUM_PRIMARYHAL );
	Aquire();	// Retrieve the current values.
	Save();		// Force a save of the unspecified values.
	return false; // On failure to load, we return false.
}

bool DirectGlideSettings::Save()
{
	// Just write the settings to the registry, who cares what was there already.
	HKEY keyDG = NULL;
	DWORD data = bFullScreen ? 1 : 0;

	// Create the registry key.
	if( RegCreateKey( HKEY_CURRENT_USER, "Software\\DirectGlide", &keyDG) != ERROR_SUCCESS )
		return false;
	
	// We have a key, so retrive the relevent info.
	if( RegSetValueEx( keyDG, "driverGUID", 0, REG_BINARY, (LPBYTE)&driverGUID, sizeof(GUID)) != ERROR_SUCCESS )
		goto FailureOnSave;
	if( RegSetValueEx( keyDG, "driverName", 0, REG_SZ, (LPBYTE)driverName, strlen(driverName)) != ERROR_SUCCESS )
		goto FailureOnSave;
	if( RegSetValueEx( keyDG, "deviceGUID", 0, REG_BINARY, (LPBYTE)&deviceGUID, sizeof(GUID)) != ERROR_SUCCESS )
		goto FailureOnSave;
	if( RegSetValueEx( keyDG, "deviceName", 0, REG_SZ, (LPBYTE)deviceName, strlen(deviceName)) != ERROR_SUCCESS )
		goto FailureOnSave;
	if( RegSetValueEx( keyDG, "ModeDescription", 0, REG_SZ, (LPBYTE)modeDesc, strlen(modeDesc)) != ERROR_SUCCESS )
		goto FailureOnSave;
	if( RegSetValueEx( keyDG, "FullScreen", 0, REG_DWORD, (LPBYTE)&data, sizeof(DWORD)) != ERROR_SUCCESS )
		goto FailureOnSave;

	RegCloseKey( keyDG );
	return true;

FailureOnSave:
	// Make sure the registry key is closed.
	if( keyDG )
		RegCloseKey( keyDG );

	return false;
}

// Aquire the settings from the enumeration system.
bool DirectGlideSettings::Aquire()
{
	D3DEnum_DriverInfo	*pDriverInfo;
	D3DEnum_DeviceInfo	*pDeviceInfo;
	D3DEnum_ModeInfo	*pModeInfo;
	D3DEnum_GetSelectedDriver(	&pDriverInfo, &pDeviceInfo );
	pDeviceInfo = pDriverInfo->pCurrentDevice;
	pModeInfo = pDeviceInfo->pCurrentMode;

	// Record the relevent info.
	memcpy( &driverGUID, &pDriverInfo->guid, sizeof(GUID));
	memcpy( driverName, pDriverInfo->strDesc, 40 );
	memcpy( &deviceGUID, &pDeviceInfo->guid, sizeof(GUID));
	memcpy( deviceName, pDeviceInfo->strName, 40 );
	memcpy( modeDesc, pModeInfo->strDesc, 40 );

	bFullScreen = (pDeviceInfo->bWindowed == TRUE) ? false : true;
	return true;
}
