// *****************************************************************************
// DGAlpha.CPP : Opperates on all grAlpha functions.
// *****************************************************************************
// DirectGlide2X                                                Official release
// Copyright (C) 1999                                           Khalid Shaikh
//                                                              Aaron Hilton
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Khalid Shaikh                                                   1255 Theberge
// kshaikh@ece.mcgill.ca                                        Brossard, Quebec
// http://www.angelic-coders.com/kshaikh (IE4 only)              Canada  J4W 2N2
//
// Please read the GNU CopyLeft License Agreement included with this code.
// Located in: LICENSE.TXT
// *****************************************************************************

#include "stdafx.h"

//-------------------------------------------------------------------
// ConvBlendFunction
//	Translate the glide blending function into
//	Direct3D's D3DBLEND format.
//-------------------------------------------------------------------
D3DBLEND ConvBlendFunction( GrAlphaBlendFnc_t blendFunc );

//-----------------------------------
// Alpha processing.
//-----------------------------------
EXPORT void STD grAlphaBlendFunction(	GrAlphaBlendFnc_t rgb_sf,
										GrAlphaBlendFnc_t rgb_df,
										GrAlphaBlendFnc_t alpha_sf,
										GrAlphaBlendFnc_t alpha_df )
{
	// Alpha blending.
	pGlideState->alpha_rgb_source	= rgb_sf;
	pGlideState->alpha_rgb_dest		= rgb_df;
	pGlideState->alpha_alpha_source	= alpha_sf;
	pGlideState->alpha_alpha_dest	= alpha_df;

	LPDIRECT3DDEVICE3 pDevice = pDTDState->Framework.GetD3DDevice();

	// Strait from Let's GliD3D wrapper... With a bit of work.
	if(rgb_sf == GR_BLEND_ONE && rgb_df == GR_BLEND_ZERO)
	{ // Generic texture blending state, no blending ops.
		pDevice->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, FALSE);
		if( pGlideState->color_local == GR_COMBINE_LOCAL_NONE )
		{
			pDevice->SetRenderState(D3DRENDERSTATE_TEXTUREMAPBLEND, D3DTBLEND_DECAL);
		} else
		{
			pDevice->SetRenderState(D3DRENDERSTATE_TEXTUREMAPBLEND, D3DTBLEND_MODULATE);
		}

		pDTDState->bUseBlending = false;
		pDevice->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, FALSE);
	} else
	{ // The rest of the blending states, do translation, etc..
		// State we're using blending now.
		pDTDState->bUseBlending = true;
		pDevice->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE);

		// Translate the blending modes.
		D3DBLEND blend_src, blend_dst;
		blend_src = ConvBlendFunction( rgb_sf );
		blend_dst = ConvBlendFunction( rgb_df );

		// Set their state info into the device.
		pDevice->SetRenderState(D3DRENDERSTATE_SRCBLEND,  blend_src);
		pDevice->SetRenderState(D3DRENDERSTATE_DESTBLEND, blend_dst);
	}

	// Say what happened, so state changes happen in the BeginScene step.
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kAlpha;
}

// Store the alpha combine state, and announce
// for next "beginScene" to update the D3D states.
EXPORT void STD grAlphaCombine(	GrCombineFunction_t func,
							GrCombineFactor_t factor,
							GrCombineLocal_t local,
							GrCombineOther_t other,
							FxBool invert )
{
	// Just copy the settings for now.
	pGlideState->alpha_func = func;
	pGlideState->alpha_factor = factor;
	pGlideState->alpha_local = local;
	pGlideState->alpha_other = other;
	pGlideState->alpha_invert = invert;

	// Say what happened, so state changes happen in the BeginScene step.
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kAlpha;
}

// Hi bit iterated RGB lighting.  FX = Self Lit texture pixels.
EXPORT void STD grAlphaControlsITRGBLighting( FxBool enable )
{
}

//-----------------------------------
// Chromakey, in the alpha stage.
//-----------------------------------
EXPORT void STD grAlphaTestFunction( GrCmpFnc_t function )
{
	LPDIRECT3DDEVICE3 pDevice = pDTDState->Framework.GetD3DDevice();

	// Store the alpha test function.
	pGlideState->alpha_test_func = function;

	// I'm so happy Let and I think alike.

	// Translate the alpha testing function, and set the testing mode.
	switch (function)
	{
		case GR_CMP_NEVER:
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE);
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, D3DCMP_NEVER);
			break;
		case GR_CMP_LESS:
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE);
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, D3DCMP_LESS);
			break;
		case GR_CMP_EQUAL:
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE);
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, D3DCMP_EQUAL);
			break;
		case GR_CMP_LEQUAL:
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE);
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, D3DCMP_LESSEQUAL);
			break;
		case GR_CMP_GREATER:
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE);
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, D3DCMP_GREATER);
			break;
		case GR_CMP_NOTEQUAL:
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE);
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, D3DCMP_NOTEQUAL);
			break;
		case GR_CMP_GEQUAL:
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, TRUE);
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, D3DCMP_GREATEREQUAL);
			break;
		case GR_CMP_ALWAYS:
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHATESTENABLE, FALSE);
			pDevice->SetRenderState(D3DRENDERSTATE_ALPHAFUNC, D3DCMP_ALWAYS);
			break;
	}
}

EXPORT void STD grAlphaTestReferenceValue( GrAlpha_t value )
{
	// Remember the reference value.
	pGlideState->alpha_test_value = value;

	// And set the device's state.
	pDTDState->Framework.GetD3DDevice()->SetRenderState(D3DRENDERSTATE_ALPHAREF, value);
}


//-------------------------------------------------------------------
// ConvBlendFunction
//	Translate the glide blending function into
//	Direct3D's D3DBLEND format.
//-------------------------------------------------------------------
D3DBLEND ConvBlendFunction( GrAlphaBlendFnc_t blendFunc )
{
	switch( blendFunc )
	{
		case GR_BLEND_ZERO:
			return D3DBLEND_ZERO;
		case GR_BLEND_ONE:
		case GR_BLEND_PREFOG_COLOR:
			return D3DBLEND_ONE;
		case GR_BLEND_DST_COLOR:
			return D3DBLEND_DESTCOLOR;
		case GR_BLEND_ONE_MINUS_DST_COLOR:
			return D3DBLEND_INVDESTCOLOR;
		case GR_BLEND_SRC_ALPHA:
			return D3DBLEND_SRCALPHA;
		case GR_BLEND_ONE_MINUS_SRC_ALPHA:
			return D3DBLEND_INVSRCALPHA;
		case GR_BLEND_DST_ALPHA:
			return D3DBLEND_DESTALPHA;
		case GR_BLEND_ONE_MINUS_DST_ALPHA:
			return D3DBLEND_INVDESTALPHA;
	}

	return D3DBLEND_ZERO;
}
