// *****************************************************************************
// DGBuffer.CPP : Opperates on all grBuffer functions.
// *****************************************************************************
// DirectGlide2X                                                Official release
// Copyright (C) 1999                                           Khalid Shaikh
//                                                              Aaron Hilton
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Khalid Shaikh                                                   1255 Theberge
// kshaikh@ece.mcgill.ca                                        Brossard, Quebec
// http://www.angelic-coders.com/kshaikh (IE4 only)              Canada  J4W 2N2
//
// Please read the GNU CopyLeft License Agreement included with this code.
// Located in: LICENSE.TXT
// *****************************************************************************

#include "stdafx.h"

// Clear the appropriate buffers.
EXPORT void STD grBufferClear( GrColor_t color, GrAlpha_t alpha, FxU16 depth )
{
	// Make sure we're not in a scene drawing state.
	pDTDState->EndScene();

	// Clear the buffer.
	DDBLTFX DDBLT;
	DDBLT.dwSize = sizeof(DDBLTFX);

	if(pGlideState->mask_rgb)
	{
		// Convert to the BLT fill color format.
		DDBLT.dwFillColor = DGColorARGB5650(color);
        pDTDState->Framework.GetRenderSurface()->Blt(NULL, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &DDBLT);
	}

	if(pGlideState->depth_mask)
	{	// Wipe the Z-Buffer.
		// TBI - Must implement a ZBuffer translation function here, if we're in
		//       WBuffering mode, or are in a different ZBuffer bit depth.
		DDBLT.dwFillDepth = depth;
		pDTDState->Framework.GetZBuffer()->Blt(NULL, NULL, NULL, DDBLT_DEPTHFILL | DDBLT_WAIT, &DDBLT);
	}

}

// Retrieve the number of pending buffer swaps.
EXPORT int STD grBufferNumPending( void )
{
	return 0; // (TBI - We're temporarily pretending to not have any pending buffers.)
}

// Swap buffers, waiting vwaits number of vertical refreshes, before doing the swap.
EXPORT void STD grBufferSwap( int vwaits )
{
	// Make sure we're not in a scene drawing state.
	pDTDState->EndScene();

	// Flip the screen.
	pDTDState->Framework.GetFrontBuffer()->Flip( NULL, DDFLIP_WAIT );
}

// Select the target rendering buffer.
EXPORT void STD grRenderBuffer( GrBuffer_t buffer )
{
	// Make sure we're not in a scene drawing state.
	pDTDState->EndScene();

	if( pGlideState->render_buffer != buffer )
	{
		pGlideState->render_buffer = buffer;

		switch(pGlideState->render_buffer)
		{
    		case GR_BUFFER_FRONTBUFFER:
				pDTDState->Framework.ChangeRenderTarget( pDTDState->Framework.GetFrontBuffer() );
				break;

    		case GR_BUFFER_BACKBUFFER:
				pDTDState->Framework.ChangeRenderTarget( pDTDState->Framework.GetBackBuffer() );
				break;
		}
	}
}
