// *****************************************************************************
// DGColor.CPP : Opperates on all grColor functions.
// *****************************************************************************
// DirectGlide2X                                                Official release
// Copyright (C) 1999                                           Khalid Shaikh
//                                                              Aaron Hilton
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Khalid Shaikh                                                   1255 Theberge
// kshaikh@ece.mcgill.ca                                        Brossard, Quebec
// http://www.angelic-coders.com/kshaikh (IE4 only)              Canada  J4W 2N2
//
// Please read the GNU CopyLeft License Agreement included with this code.
// Located in: LICENSE.TXT
// *****************************************************************************

#include "stdafx.h"

// Color conversion functions.
DWORD (*DGColorARGB8888)(DWORD &dw8888);
DWORD (*DGColorARGB5650)(DWORD &dw8888);

//-------------------------------------------------------------------
// Color conversion macro functions. (Thanks to Khalid!)
//-------------------------------------------------------------------
// Convert to 8888 color format (Direct3D colors)
DWORD ARGB8888toARGB8888(DWORD &dwARGB8888)
{
	return dwARGB8888;
}

DWORD ABGR8888toARGB8888(DWORD &dwABGR8888)
{
	return	((dwABGR8888)    &0xFF000000) |    // A
        	((dwABGR8888<<16)&0x00FF0000) |    // R
        	((dwABGR8888)    &0x0000FF00) |    // G
        	((dwABGR8888>>16)&0x000000FF);     // B
}

DWORD RGBA8888toARGB8888(DWORD &dwRGBA8888)
{
	return	(dwRGBA8888>>8) |
			(dwRGBA8888<<24);
}

DWORD BGRA8888toARGB8888(DWORD &dwBGRA8888)
{
	return	((dwBGRA8888<<24)&0xFF000000) |  // A
        	((dwBGRA8888<<8) &0x00FF0000) |  // R
        	((dwBGRA8888>>8) &0x0000FF00) |  // G
        	((dwBGRA8888<<24)&0x000000FF);   // B
}

//-------------------------------------------------------------------
// Color conversion macro functions. (Thanks to Khalid!)
//-------------------------------------------------------------------
// Convert to 5650 color format (SurfaceDIB colors)
DWORD ARGB8888toRGBA5650(DWORD &dwARGB8888)
{
	DWORD R = (dwARGB8888>>(16+3))&0x1F;
	DWORD G = (dwARGB8888>>(8+2))&0x3F;
	DWORD B = (dwARGB8888>>(3))&0x1F;
	return (R<<11) | (G<<5) | B;
}

DWORD ABGR8888toRGBA5650(DWORD &dwABGR8888)
{
	DWORD B = (dwABGR8888>>(16+3))&0x1F;
	DWORD G = (dwABGR8888>>(8+2))&0x3F;
	DWORD R = (dwABGR8888>>(3))&0x1F;
	return (R<<11) | (G<<5) | B;
}

DWORD RGBA8888toRGBA5650(DWORD &dwRGBA8888)
{
	DWORD R = (dwRGBA8888>>(24+3))&0x1F;
	DWORD G = (dwRGBA8888>>(16+2))&0x3F;
	DWORD B = (dwRGBA8888>>(8+3))&0x1F;
	return (R<<11) | (G<<5) | B;
}

DWORD BGRA8888toRGBA5650(DWORD &dwBGRA8888)
{
	DWORD B = (dwBGRA8888>>(24+3))&0x1F;
	DWORD G = (dwBGRA8888>>(16+2))&0x3F;
	DWORD R = (dwBGRA8888>>(8+3))&0x1F;
	return (R<<11) | (G<<5) | B;
}

//-------------------------------------------------------------------
// Color combine processing.
EXPORT void STD grColorCombine(	GrCombineFunction_t func,
							GrCombineFactor_t factor,
							GrCombineLocal_t local,
							GrCombineOther_t other,
							FxBool invert )
{
	// Just copy the settings for now.
	pGlideState->color_func = func;
	pGlideState->color_factor = factor;
	pGlideState->color_local = local;
	pGlideState->color_other = other;
	pGlideState->color_invert = invert;

	// Say what happened, so state changes happen in the BeginScene step.
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kColor;
}

// Write masking enable/disable. (ColorMask, and AlphaMask)
EXPORT void STD grColorMask( FxBool rgb, FxBool alpha )
{
	// DX6 docs say:
	// To disable writes to the color buffer by using alpha blending,
	// you can set D3DRENDERSTATE_SRCBLEND to D3DBLEND_ZERO and
	// D3DRENDERSTATE_DESTBLEND to D3DBLEND_ONE.

	if( rgb == FXFALSE )
	{
		if( pGlideState->mask_rgb != rgb )
		{
			pDTDState->Framework.GetD3DDevice()->SetRenderState( D3DRENDERSTATE_SRCBLEND, D3DBLEND_ZERO );
			pDTDState->Framework.GetD3DDevice()->SetRenderState( D3DRENDERSTATE_DESTBLEND, D3DBLEND_ONE );
		}
	} else {
		if( pGlideState->mask_rgb != rgb )
		{
			pDTDState->Framework.GetD3DDevice()->SetRenderState( D3DRENDERSTATE_SRCBLEND, D3DBLEND_ONE );
			pDTDState->Framework.GetD3DDevice()->SetRenderState( D3DRENDERSTATE_DESTBLEND, D3DBLEND_ZERO );
		}
	}

	pGlideState->mask_rgb = rgb;
	pGlideState->mask_alpha = alpha;

	// Say what happened, so state changes happen in the BeginScene step.
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kMask;

	// TBI - Can we actually support alpha destination (auxilary buffer) blending?
}

// Constant color value.
EXPORT void STD grConstantColorValue( GrColor_t color )
{
	pGlideState->color_value = color;
	pDTDState->dwConstantColor = (DGColorARGB8888)(color);
	pDTDState->dwConstantColorNoAlpha = pDTDState->dwConstantColor&0x00FFFFFF;
	pDTDState->dwConstantAlpha = pDTDState->dwConstantColor >> 24;
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kColor;
}

// Translate component color values, into the current constant color value format, and stores it.
EXPORT void STD grConstantColorValue4( float a, float r, float g, float b )
{
	int iA, iR, iG, iB;
	iA = (int)(a*255.49f);
	iR = (int)(r*255.49f);
	iG = (int)(g*255.49f);
	iB = (int)(b*255.49f);

	// Reverse convert the color values to Glide's current supported format.
	switch( pGlideState->color_format )
	{
		case GR_COLORFORMAT_ARGB:
			pGlideState->color_value = (iA<<24)+(iR<<16)+(iG<<8)+(iB<<8);
			break;

		case GR_COLORFORMAT_ABGR:
			pGlideState->color_value = (iA<<24)+(iB<<16)+(iG<<8)+(iR<<8);
			break;

		case GR_COLORFORMAT_RGBA:
			pGlideState->color_value = (iR<<24)+(iG<<16)+(iB<<8)+(iA<<8);
			break;

		case GR_COLORFORMAT_BGRA:
			pGlideState->color_value = (iB<<24)+(iG<<16)+(iR<<8)+(iA<<8);
			break;
	}

	// Use the generic setting function to actually remember the new color value.
	grConstantColorValue( pGlideState->color_format );
}
