// *****************************************************************************
// DGDraw.CPP : Opperates on all grDraw functions.
// *****************************************************************************
// DirectGlide2X                                                Official release
// Copyright (C) 1999                                           Khalid Shaikh
//                                                              Aaron Hilton
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Khalid Shaikh                                                   1255 Theberge
// kshaikh@ece.mcgill.ca                                        Brossard, Quebec
// http://www.angelic-coders.com/kshaikh (IE4 only)              Canada  J4W 2N2
//
// Please read the GNU CopyLeft License Agreement included with this code.
// Located in: LICENSE.TXT
// *****************************************************************************

#include "stdafx.h"
#include "VertexProcessor.h"

// Prepare the snapping vertex processor based on the first vertex.
#define PrepSnap( x ) \
{ \
	if( x > 2048 ) \
	{ \
		if( pDTDState->bFlipY == false) \
		{ \
			VP_Snap = VP_P_Snap; \
		} else \
		{ \
			VP_Snap = VP_P_Snap_Flip; \
		} \
	} else \
	{ \
		if( pDTDState->bFlipY == false) \
		{ \
			VP_Snap = VP_P_NoSnap; \
		} else \
		{ \
			VP_Snap = VP_P_NoSnap_Flip; \
		} \
	} \
}


EXPORT void STD grDrawLine( const GrVertex *a, const GrVertex *b )
{
	// Make sure we have begun our scene drawing state.
	pDTDState->BeginScene();

	// Configure the vertex processor based on the first vertex.
	PrepSnap( a->x )

	// Process the two vertices.
	VertexProcessor( a, 0);
	VertexProcessor( b, 1);

	// Draw the line.
	pDTDState->Framework.GetD3DDevice()->DrawPrimitive( D3DPT_LINELIST, D3DFVF_TLVERTEX, vertexBuffer, 2, D3DDP_DONOTCLIP | D3DDP_DONOTLIGHT );
}

EXPORT void STD grDrawPlanarPolygon( int nVerts, int iList[], const GrVertex vList[] )
{
	// Make sure we have begun our scene drawing state.
	pDTDState->BeginScene();

	// Configure the vertex processor based on the first vertex.
	PrepSnap( vList[0].x )

	// Process all the vertices in the indexed vertex list.
	for( int i=0; i<nVerts; i++ )
	{
		VertexProcessor( &vList[iList[i]], i );
	}

	// Draw the polygon as a Triangle Fan.
	pDTDState->Framework.GetD3DDevice()->DrawPrimitive( D3DPT_TRIANGLEFAN, D3DFVF_TLVERTEX, vertexBuffer, nVerts, D3DDP_DONOTCLIP | D3DDP_DONOTLIGHT );
}

EXPORT void STD grDrawPolygon( int nVerts, int iList[], const GrVertex vList[] )
{
	// Make sure we have begun our scene drawing state.
	pDTDState->BeginScene();

	// Configure the vertex processor based on the first vertex.
	PrepSnap( vList[0].x )

	// Process all the vertices in the indexed vertex list.
	for( int i=0; i<nVerts; i++ )
	{
		VertexProcessor( &vList[iList[i]], i );
	}

	// Draw the polygon as a Triangle Fan.
	pDTDState->Framework.GetD3DDevice()->DrawPrimitive( D3DPT_TRIANGLEFAN, D3DFVF_TLVERTEX, vertexBuffer, nVerts, D3DDP_DONOTCLIP | D3DDP_DONOTLIGHT );
}

EXPORT void STD grDrawPlanarPolygonVertexList( int nVerts, const GrVertex vList[] )
{
	// Make sure we have begun our scene drawing state.
	pDTDState->BeginScene();

	// Configure the vertex processor based on the first vertex.
	PrepSnap( vList[0].x )

	// Process the vertex list.
	for( int i=0; i<nVerts; i++ )
	{
		VertexProcessor( &vList[i], i );
	}

	// Draw the polygon as a Triangle Fan.
	pDTDState->Framework.GetD3DDevice()->DrawPrimitive( D3DPT_TRIANGLEFAN, D3DFVF_TLVERTEX, vertexBuffer, nVerts, D3DDP_DONOTCLIP | D3DDP_DONOTLIGHT );
}

EXPORT void STD grDrawPolygonVertexList( int nVerts, const GrVertex vList[] )
{
	// Make sure we have begun our scene drawing state.
	pDTDState->BeginScene();

	// Configure the vertex processor based on the first vertex.
	PrepSnap( vList[0].x )

	// Process the vertex list.
	for( int i=0; i<nVerts; i++ )
	{
		VertexProcessor( &vList[i], i );
	}

	// Draw the polygon as a Triangle Fan.
	pDTDState->Framework.GetD3DDevice()->DrawPrimitive( D3DPT_TRIANGLEFAN, D3DFVF_TLVERTEX, vertexBuffer, nVerts, D3DDP_DONOTCLIP | D3DDP_DONOTLIGHT );
}

EXPORT void STD grDrawPoint( const GrVertex *a )
{
	// Make sure we have begun our scene drawing state.
	pDTDState->BeginScene();

	// Configure the vertex processor based on the first vertex.
	PrepSnap( a->x );

	// Process the single vertex.
	VertexProcessor( a, 0 );

	// Draw the dot.
	pDTDState->Framework.GetD3DDevice()->DrawPrimitive( D3DPT_POINTLIST, D3DFVF_TLVERTEX, vertexBuffer, 1, D3DDP_DONOTCLIP | D3DDP_DONOTLIGHT );
}

EXPORT void STD grDrawTriangle( const GrVertex *a, const GrVertex *b, const GrVertex *c )
{
	// Make sure we have begun our scene drawing state.
	pDTDState->BeginScene();

	// Configure the vertex processor based on the first vertex.
	PrepSnap( a->x );

	// Process the three vertices.
	VertexProcessor( a, 0);
	VertexProcessor( b, 1);
	VertexProcessor( c, 2);

	// Draw the triangle.
	pDTDState->Framework.GetD3DDevice()->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX, vertexBuffer, 3, D3DDP_DONOTCLIP | D3DDP_DONOTLIGHT );
}
