// *****************************************************************************
// DGSst.CPP : Opperates on all grSst functions.
// *****************************************************************************
// DirectGlide2X                                                Official release
// Copyright (C) 1999                                           Khalid Shaikh
//                                                              Aaron Hilton
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Khalid Shaikh                                                   1255 Theberge
// kshaikh@ece.mcgill.ca                                        Brossard, Quebec
// http://www.angelic-coders.com/kshaikh (IE4 only)              Canada  J4W 2N2
//
// Please read the GNU CopyLeft License Agreement included with this code.
// Located in: LICENSE.TXT
// *****************************************************************************

#include "stdafx.h"


// enable/disable the passthrough feature.
EXPORT FxBool STD grSstControl( FxU32 code )
{
	return FXTRUE;
}

// Wait until hardware is idle.
EXPORT void STD grSstIdle( void )
{
}

// Hardware busy?  Never.
EXPORT FxBool STD grSstIsBusy( void )
{
	return FXFALSE;
}

// Origin info, UpperLeft (going down) or LowerLeft (going up).
EXPORT void STD grSstOrigin( GrOriginLocation_t origin )
{
	// Only flip the origin when needed.
	if( pGlideState->origin_location != origin )
	{
		pGlideState->origin_location = origin;

		if( origin == GR_ORIGIN_UPPER_LEFT )
		{ // Deal with the usual case of having an upper left coordinate system. (Computer Graphics)
			pDTDState->bFlipY = false;
		} else
		{ // Deal with the odd-ball case of having a flipped coordinate system. (Mathematical Origin)
			pDTDState->bFlipY = true;
		}
	}
}

// Performance statistics.
EXPORT void STD grSstPerfStats( GrSstPerfStats_t *pStats )
{
}

// Reset the performance statistics.
EXPORT void STD grSstResetPerfStats( void )
{
}

// Find out how many SST subsystem exist in this system. (only one)
EXPORT FxBool STD grSstQueryBoards( GrHwConfiguration *HwConfig )
{
	memset(HwConfig, 0, sizeof(GrHwConfiguration));
	HwConfig->num_sst = 1;
	return FXTRUE;
}

// Describe what the hell this wrapper really does.
EXPORT FxBool STD grSstQueryHardware( GrHwConfiguration *HwConfig )
{
	// set the card up as a Voodoo 1 with 2MB of frame buffer
	// and 2MB of texture ram
	HwConfig->num_sst = 1;
	HwConfig->SSTs[0].type = GR_SSTTYPE_VOODOO;
	HwConfig->SSTs[0].sstBoard.VoodooConfig.fbRam = 2;
	HwConfig->SSTs[0].sstBoard.VoodooConfig.fbiRev = 2;
	HwConfig->SSTs[0].sstBoard.VoodooConfig.nTexelfx = 1;
	HwConfig->SSTs[0].sstBoard.VoodooConfig.sliDetect = FALSE;
	HwConfig->SSTs[0].sstBoard.VoodooConfig.tmuConfig[0].tmuRev = 1;
	HwConfig->SSTs[0].sstBoard.VoodooConfig.tmuConfig[0].tmuRam = 2;

	return FXTRUE;
}

// Height in pixels of the hardware's screen.
EXPORT FxU32 STD grSstScreenHeight( void )
{
	// TBI - whatever.
	return 480;
}

// Width in pixels of the hardware's screen.
EXPORT FxU32 STD grSstScreenWidth( void )
{
	// TBI - whatever.
	return 640;
}

// Select a subsystem.  But, this ain't gonna be implemented any time soon.
EXPORT void STD grSstSelect( int which_sst )
{
}

// SST Status register info.
EXPORT FxU32 STD grSstStatus( void )
{
	return 0;
}

// Current line the beam is drawing.
EXPORT FxU32 STD grSstVideoLine( void )
{
	return 0;
}

// Vertical retrace wait status.
EXPORT FxBool STD grSstVRetraceOn( void )
{
	return FXTRUE;
}

// Close up the Sst graphics system.
EXPORT void STD grSstWinClose( void )
{
	// Finish our scene rendering procedure.
	pDTDState->EndScene();
	pDTDState->Framework.DestroyObjects();
}

//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Callback - Called during device intialization, this code checks the device
//       for some minimum set of capabilities.
//-----------------------------------------------------------------------------
HRESULT ConfirmDevice( DDCAPS* pddDriverCaps, D3DDEVICEDESC* pd3dDeviceDesc );

//-----------------------------------------------------------------------------
// Name: Initialize3DEnvironment()
// Desc: Initializes the sample framework, then calls the app-specific function
//       to initialize device specific objects. This code is structured to
//       handled any errors that may occur duing initialization
//-----------------------------------------------------------------------------
bool Init3DEnvironment( HWND hWnd, DWORD dwFrameworkFlags, DDSURFACEDESC2* ddsd );

//-----------------------------------------------------------------------------
// DESCRIPTION:
//	Used by GlideWindow. Proc to be used with EnumThreadWindows in grSstWinOpen.
//	1/16/99
//-----------------------------------------------------------------------------
static BOOL CALLBACK enumProc(HWND hwnd, LPARAM lParam)
{
	pDTDState->hParentWnd = hwnd;
	return TRUE;
}

// Message processing for DirectGlide window.
long __stdcall WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);



// Open the Sst graphics system.
EXPORT FxBool STD grSstWinOpen(	FxU32 hGLWnd,
							GrScreenResolution_t screen_resolution,
							GrScreenRefresh_t refresh_rate,
							GrColorFormat_t color_format,
							GrOriginLocation_t origin_location,
							int nColBuffers,
							int nAuxBuffers)
{
	HWND hWnd = (HWND)hGLWnd;
	DWORD dwWidth, dwHeight;

	// Grab the screen resolution settings.
	ConvertScreenRes( screen_resolution, dwWidth, dwHeight );
	if(( dwWidth == 0 ) || ( dwHeight == 0 ))
	{
		dwWidth = 640;
		dwHeight = 480;
	}

	// Make sure we have valid buffer parameters, with what this wrapper can take.
	if(nColBuffers > 2) return FXFALSE;
	if(nAuxBuffers > 1) return FXFALSE;

	//-----------------------------------
	// Set up the color transform macro functions.
	switch(color_format)
	{
		case GR_COLORFORMAT_ARGB:
			DGColorARGB8888 = ARGB8888toARGB8888;
			DGColorARGB5650 = ARGB8888toRGBA5650;
			break;

		case GR_COLORFORMAT_ABGR:
			DGColorARGB8888 = ABGR8888toARGB8888;
			DGColorARGB5650 = ABGR8888toRGBA5650;
			break;

		case GR_COLORFORMAT_RGBA:
			DGColorARGB8888 = RGBA8888toARGB8888;
			DGColorARGB5650 = RGBA8888toRGBA5650;
			break;

		case GR_COLORFORMAT_BGRA:
			DGColorARGB8888 = BGRA8888toARGB8888;
			DGColorARGB5650 = BGRA8888toRGBA5650;
			break;

		default:
			Assert(false); // Unhandled color format.
	}

	//-----------------------------------
	// Enumerate available D3D devices, passing a callback that allows devices
	// to be accepted/rejected based on what capabilities the app requires.
	HRESULT hr;
	if( FAILED( hr = D3DEnum_EnumerateDevices( ConfirmDevice ) ) )
	{
        return FXFALSE;
	}

	//-----------------------------------
	// Load our configuration settings from the registry.
	if( !dgs.Load() )
	{
        return FXFALSE;
	}

	//-----------------------------------
	// Either they gave us an hWnd, if not we get the hWnd of the active window
	if(!hWnd) hWnd = GetActiveWindow();

	if(( !hWnd ) || (!dgs.bFullScreen))
	{
		// GlideWindow:
		// This initializes the rendering window. It's slightly modified from
		// what Khalid conveniently left behind. 1/16/99
		if (EnumThreadWindows(GetCurrentThreadId(), (WNDENUMPROC) enumProc, NULL)==0)
		return FALSE;

		// update: if you arn't upgraded to Unreal 220, you need the following code to
		// even run unreal.
		// (you also need it for glide windowed rendering now. 1/16/99)
		HINSTANCE hInst = (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE);
		WNDCLASS wndclass;
		wndclass.style = CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc = WindowProc;
		wndclass.cbClsExtra = 0;
		wndclass.cbWndExtra = 0;
		wndclass.hInstance = hInst;
		wndclass.hIcon = NULL;
		wndclass.hCursor = NULL;
		wndclass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
		wndclass.lpszMenuName = NULL;
		wndclass.lpszClassName = "wndclass";
		RegisterClass(&wndclass);

		hWnd = CreateWindow("wndclass", "DirectGlide", WS_OVERLAPPED, 0, 0, dwWidth, dwHeight, NULL, NULL, hInst, NULL);
		ShowWindow(hWnd, SW_SHOWNORMAL);
		UpdateWindow(hWnd);

		if(!hWnd) return FXFALSE;
	}

	pDTDState->hWnd = (HWND)hWnd;                      // set the global variable
	pGlideState->screen_resolution = screen_resolution;
	pGlideState->color_format	= color_format;
	pGlideState->origin_location= origin_location;
	pGlideState->nColBuffers	= nColBuffers;
	pGlideState->nAuxBuffers	= nAuxBuffers;

	//-----------------------------------
	// Initialize the Direct3D framework.
	// Setup the framework flags.
    DWORD   dwFrameworkFlags = 0L;
	dwFrameworkFlags |= D3DFW_NO_FPUSETUP;
    dwFrameworkFlags |= ( dgs.bFullScreen	? D3DFW_FULLSCREEN : 0L );
    dwFrameworkFlags |= ( nAuxBuffers		? D3DFW_ZBUFFER    : 0L );
    dwFrameworkFlags |= ( (nColBuffers>1)	? D3DFW_BACKBUFFER : 0L );

	// Specify our surface properties.

	// Find the device, and driver selected in our registry.
	D3DEnum_DriverInfo* pDriver = D3DEnum_GetFirstDriver();

	// Scan through all the 3D cards.
	bool bGotIt = false;
	while( pDriver && !bGotIt )
	{
		if( memcmp(&pDriver->guid, &dgs.driverGUID, sizeof(GUID) ) == 0 )
			bGotIt = true;
		else
			pDriver = pDriver->pNext;
	}

	// Make sure we have a valid driver.
	if( !pDriver )
		return FXFALSE;

	// Found the Driver, now scan through each device.
	D3DEnum_DeviceInfo* pDevice = pDriver->pFirstDevice;
	bGotIt = false;
	while( pDevice && !bGotIt )
	{
		if( memcmp(&pDevice->guid, &dgs.deviceGUID, sizeof(GUID) ) == 0 )
			bGotIt = true;
		else
			pDevice = pDevice->pNext;
	}

	// Make sure we have a valid device.
	if( !pDriver )
		return FXFALSE;

	// For full screen we need a valid mode description.
	D3DEnum_ModeInfo* pMode = pDevice->pFirstMode;
	if( dgs.bFullScreen )
	{
		// Found the device, and we're in Fullscreen, now scan through each mode.
		bGotIt = false;
		while( pMode && !bGotIt )
		{
			if( strcmp(pMode->strDesc, dgs.modeDesc) == 0 )
				bGotIt = true;
			else
				pMode = pMode->pNext;
		}

		// Make sure we have a valid video mode.
		if( !pMode )
			return FXFALSE;
	}

	// Initialize the environment, and make sure everything's cool.
	if( FAILED( Init3DEnvironment( pDTDState->hWnd, dwFrameworkFlags, &pMode->ddsd )))
		return FXFALSE;

	// O.k.. We got the 3D environment setup, but we must adjust the viewport
	//  to translate the Glide coordinates to the resolution of the display.
	D3DVIEWPORT2 Viewport;
	Viewport.dwSize = sizeof(D3DVIEWPORT2);
	pDTDState->Framework.GetViewport()->GetViewport2( &Viewport );

	// TBI - Determine if Window mode really screws with the vertex positioning in the
	// final coordinates, if so we must insert the correct translation coordinates here.
	pDTDState->fTranslateX = 0;
	pDTDState->fTranslateY = 0;

	// Setup the vertex scaling for the target viewport resolution.
	pDTDState->fScaleX=(float)Viewport.dwWidth/(float)dwWidth;  // TargetWidth/GlideWidth = scaling factor.
	pDTDState->fScaleY=(float)Viewport.dwHeight/(float)dwHeight;  // TargetHeight/GlideHeight = scaling factor.

	if( origin_location == GR_ORIGIN_UPPER_LEFT )
	{ // Deal with the usual case of having an upper left coordinate system. (Computer Graphics)
		pDTDState->bFlipY = false;
	} else
	{ // Deal with the odd-ball case of having a flipped coordinate system. (Mathematical Origin)
		pDTDState->bFlipY = true;
	}

	pDTDState->fViewportWidth = (float)dwWidth;
	pDTDState->fViewportHeight = (float)dwHeight;

	// Define the clipping volume for the viewport.
	Viewport.dvClipX = 0;    // Testing centered coordinates.
    Viewport.dvClipY = 0;
    Viewport.dvClipWidth = pDTDState->fViewportWidth;
    Viewport.dvClipHeight = pDTDState->fViewportHeight;
	Viewport.dvMinZ = 0;
	Viewport.dvMaxZ = 1.f;

	pDTDState->Framework.GetViewport()->SetViewport2( &Viewport );
	pDTDState->Framework.GetD3DDevice()->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFN_POINT);
	pDTDState->Framework.GetD3DDevice()->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_LINEAR);

	pDTDState->Framework.GetD3DDevice()->SetTextureStageState( 0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
	pDTDState->Framework.GetD3DDevice()->SetTextureStageState( 0, D3DTSS_ADDRESSV, D3DTADDRESS_CLAMP);
	pDTDState->PrepareTextureModes();

	// Make sure the scene state flag is initialized correctly.
	pDTDState->bSceneReady = false;
	return FXTRUE;
}


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Callback - Called during device intialization, this code checks the device
//       for some minimum set of capabilities.
//-----------------------------------------------------------------------------
HRESULT ConfirmDevice( DDCAPS* pddDriverCaps, D3DDEVICEDESC* pd3dDeviceDesc )
{
    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: ConvertScreenRes()
// Desc: Utility function - fills the dwHeight and dwWidth variables with the
//		 screen resolution specified by "res".
//-----------------------------------------------------------------------------
void ConvertScreenRes( GrScreenResolution_t res, DWORD &dwWidth, DWORD &dwHeight )
{
	switch(res)
	{
		case GR_RESOLUTION_320x200:		dwWidth = 320;	dwHeight = 200; break;
		case GR_RESOLUTION_320x240:		dwWidth = 320;	dwHeight = 240; break;
		case GR_RESOLUTION_400x256:		dwWidth = 400;	dwHeight = 256; break;
		case GR_RESOLUTION_512x384:		dwWidth = 512;	dwHeight = 384; break;
		case GR_RESOLUTION_640x200:		dwWidth = 640;	dwHeight = 200; break;
		case GR_RESOLUTION_640x350:		dwWidth = 640;	dwHeight = 350; break;
		case GR_RESOLUTION_640x400:		dwWidth = 640;	dwHeight = 400; break;
		case GR_RESOLUTION_640x480:		dwWidth = 640;	dwHeight = 480; break;
		case GR_RESOLUTION_800x600:		dwWidth = 800;	dwHeight = 600; break;
		case GR_RESOLUTION_960x720:		dwWidth = 960;	dwHeight = 720; break;
		case GR_RESOLUTION_856x480:		dwWidth = 856;	dwHeight = 480; break;
		case GR_RESOLUTION_512x256:		dwWidth = 512;	dwHeight = 256; break;
		case GR_RESOLUTION_1024x768:	dwWidth = 1024;	dwHeight = 768; break;
		case GR_RESOLUTION_1280x1024:	dwWidth = 1280;	dwHeight = 1024;break;
		case GR_RESOLUTION_1600x1200:	dwWidth = 1600;	dwHeight = 1200;break;
		case GR_RESOLUTION_400x300:		dwWidth = 400;	dwHeight = 300; break;
		case GR_RESOLUTION_NONE:		dwWidth = 0;	dwHeight = 0; break;
	}
}

//-----------------------------------------------------------------------------
// Name: ConvertRefreshRate()
// Desc: Utility function - returns the translated refresh rate.
//-----------------------------------------------------------------------------
DWORD ConvertRefreshRate( GrScreenRefresh_t refresh )
{
	switch( refresh )
	{
		case GR_REFRESH_60Hz:	return 60;
		case GR_REFRESH_70Hz:	return 70;
		case GR_REFRESH_72Hz:	return 72;
		case GR_REFRESH_75Hz:	return 75;
		case GR_REFRESH_80Hz:	return 80;
		case GR_REFRESH_85Hz:	return 85;
		case GR_REFRESH_90Hz:	return 90;
		case GR_REFRESH_100Hz:	return 100;
		case GR_REFRESH_120Hz:	return 120;
		case GR_REFRESH_NONE:	return 0;
	}
	return 0;
}



//-----------------------------------------------------------------------------
// Name: Initialize3DEnvironment()
// Desc: Initializes the Direct3D framework.
//-----------------------------------------------------------------------------
bool Init3DEnvironment( HWND hWnd, DWORD dwFrameworkFlags, DDSURFACEDESC2* pddsd )
{
	// Initialize the D3D framework
    if( SUCCEEDED( pDTDState->Framework.Initialize( hWnd, &dgs.driverGUID,
		               &dgs.deviceGUID, pddsd, dwFrameworkFlags ) ) )
	{
		return true;
	} else
		pDTDState->Framework.DestroyObjects();

	// If we get here, the first initialization passed failed.
	// Try again using a software rasterizer instead.
	D3DEnum_DriverInfo* pDriverInfo;
	D3DEnum_DeviceInfo* pDeviceInfo;
	D3DEnum_SelectDefaultDriver( D3DENUM_SOFTWAREONLY );
    D3DEnum_GetSelectedDriver( &pDriverInfo, &pDeviceInfo );

	// Initialize the D3D framework
    if( SUCCEEDED( pDTDState->Framework.Initialize( hWnd, &pDriverInfo->guid,
		               &pDeviceInfo->guid, pddsd, dwFrameworkFlags ) ) )
	{
		return true;
	}

	return false;

}

////////////////////////////////////////////////////////////////////////////////
// DESCRIPTION:
//
//
// AUTHOR:
//    Modified by Khalid Shaikh (kshaikh@ece.mcgill.ca) on 01/09/99
//	1/16/99
//    Modified to work better with GlideWindow (transfers keboard presses to
//		the parent window which initialized glide).
////////////////////////////////////////////////////////////////////////////////
long __stdcall WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId;
	int wmEvent;

	switch (message) {
	case WM_SYSKEYUP:
	case WM_SYSKEYDOWN:
	case WM_KEYUP:
	case WM_KEYDOWN:
		PostMessage(pDTDState->hParentWnd, message, wParam, lParam);
		break;

	case WM_COMMAND:
		wmId = LOWORD(wParam);
		wmEvent = HIWORD(wParam);

		switch (wmId) {
		case 0:
		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
		}

	case WM_DESTROY:
      PostQuitMessage(0);
      break;

	default:
		return (DefWindowProc(hWnd, message, wParam, lParam));
	}

	return 0;
}
