// *****************************************************************************
// DGTex.CPP : Opperates on all grTex functions.
// *****************************************************************************
// DirectGlide2X                                                Official release
// Copyright (C) 1999                                           Khalid Shaikh
//                                                              Aaron Hilton
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Khalid Shaikh                                                   1255 Theberge
// kshaikh@ece.mcgill.ca                                        Brossard, Quebec
// http://www.angelic-coders.com/kshaikh (IE4 only)              Canada  J4W 2N2
//
// Please read the GNU CopyLeft License Agreement included with this code.
// Located in: LICENSE.TXT
// *****************************************************************************
#include "stdafx.h"

// Return the space occupied by a texture.
EXPORT FxU32 STD grTexTextureMemRequired( FxU32 evenOdd, GrTexInfo *info );

//-----------------------------------
// DESCRIPTION:																												Prototyped
//    Calculates the texture memory required by the texture
//    I'm simply ignoring the texture size for now..  Verified using EmptyDLL
//		This function does spit out the correct number.
//
// AUTHOR:
//	Modified by Khalid Shaikh (kshaikh@ece.mcgill.ca) on 01/09/99
//	Revised by Aaron Hilton (VideoGamer@home.com) on 02/14/99
//-----------------------------------
EXPORT FxU32 STD grTexCalcMemRequired(  GrLOD_t lodmin, GrLOD_t lodmax, GrAspectRatio_t aspect, GrTextureFormat_t format)
{
	// Loop through the LOD levels, calculate the corresponding texture sizes, and add them up.
	int nWidth, nHeight, nByte;
	int nTotalBytes = 0;
	for(int i=lodmax; i<=lodmin; i++)
	{
		GetTexInfo(i, aspect, format, nWidth, nHeight, nByte);
		nTotalBytes += nWidth*nHeight*nByte;
	}

	// Snap the size to the next highest eight (8) byte multiple.
	nTotalBytes += (nTotalBytes%8 > 0 ? 8-nTotalBytes%8 : 0);

	//DebugOut("!!%d %d %d %d %d!!\n", smallLod, largeLod, aspectRatio, format, nTotalBytes);
	return nTotalBytes;
}

// Texture clamping.
EXPORT void STD grTexClampMode( GrChipID_t tmu, GrTextureClampMode_t s_clampmode, GrTextureClampMode_t t_clampmode )
{
	// Determine a change in the horizontal clamp mode state.
	if( s_clampmode != pGlideState->tex_sClampMode )
	{
		pGlideState->tex_sClampMode = s_clampmode;

		D3DTEXTUREADDRESS mode;
		switch( s_clampmode )
		{
			case GR_TEXTURECLAMP_CLAMP:
				mode = D3DTADDRESS_CLAMP;
				break;

			case GR_TEXTURECLAMP_WRAP:
				mode = D3DTADDRESS_WRAP;
				break;
		}

		// Set the new horizontal texture addressing mode.
		pDTDState->Framework.GetD3DDevice()->SetTextureStageState( 0, D3DTSS_ADDRESSU, mode);
	}

	// Determine a change in the vertical clamp mode state.
	if( t_clampmode != pGlideState->tex_tClampMode )
	{
		pGlideState->tex_tClampMode = t_clampmode;

		D3DTEXTUREADDRESS mode;
		switch( t_clampmode )
		{
			case GR_TEXTURECLAMP_CLAMP:
				mode = D3DTADDRESS_CLAMP;
				break;

			case GR_TEXTURECLAMP_WRAP:
				mode = D3DTADDRESS_WRAP;
				break;
		}

		// Set the new vertical texture addressing mode.
		pDTDState->Framework.GetD3DDevice()->SetTextureStageState( 0, D3DTSS_ADDRESSV, mode);
	}
}

// Texture combine.
EXPORT void STD grTexCombine(	GrChipID_t tmu, GrCombineFunction_t rgb_function, GrCombineFactor_t rgb_factor,
								GrCombineFunction_t alpha_function, GrCombineFactor_t alpha_factor,
								FxBool rgb_invert, FxBool alpha_invert)
{
}

// Texture combine function.
EXPORT void STD grTexCombineFunction( GrChipID_t tmu, GrTextureCombineFnc_t fnc )
{
}

// Set the detail texturing mode.
EXPORT void STD grTexDetailControl( GrChipID_t tmu, int lodBias, FxU8 detailScale, float detailMax )
{
}

// Transfer a complete mipmap to the texture memory of the 3D card. (Track and Trash management method)
EXPORT void STD grTexDownloadMipMap( GrChipID_t tmu, FxU32 startAddress, FxU32 evenOdd, GrTexInfo *pInfo )
{
	Assert( pInfo ); // Valid texture info pointer.

	DWORD nTexSize = grTexTextureMemRequired( evenOdd, pInfo );
	Assert( nTexSize ); // Have something to transfer.

	// Validate the texture address is in a correct location.
	if( (startAddress+nTexSize) >= pGlideState->nMaxTextureMemory )
	{
		DebugOut("Bad DownloadMipMap start address.\n");
		return;
	}

	// Immediately transfer the relevant texture info into our virtual texture memory.
	memcpy( &pGlideState->pTexMemory[startAddress], pInfo->data, nTexSize );

	// Make sure we're not in a scene drawing state.
	// It could be bad if the texture class performs a surface lock, and if the 3D card doesn't
	// allow that when in the middle of a scene.
	pDTDState->EndScene();

	// Get a valid texture which starts at the same startAddress.
	VTexture* pTexture = pDTDState->GetTextureAt(startAddress);

	// Make sure the textures match.
	if( pTexture )
	{
		if( pTexture->IsEqual( pInfo ) )
		{	// Textures are equal here, so just notify the texture
			// info is dirty, and needs updating.
			pTexture->Update();
		}
	}
	// Relocated the actual texture creation to the grTexSource routine.
	//else
	//{
	//	// Create the new texture.
	//	// Automatically trashes the textures in the given range.
	//	pDTDState->CreateAt( startAddress, startAddress+nTexSize-1, pInfo );
	//}


	// Announce the texture settings are dirty.
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kTexture;
}

// Transfer a single mipmap to the texture memory of the 3D card.  (Track and Trash management method)
EXPORT void STD grTexDownloadMipMapLevel(	GrChipID_t tmu, FxU32 startAddress,
											GrLOD_t thisLod, GrLOD_t largeLod,
											GrAspectRatio_t aspectRatio, GrTextureFormat_t format,
											FxU32 evenOdd, void *data )
{
}

// Transfer part of a single mipmap level to the texture memory of the 3D card.  (Track and Trash management method)
EXPORT void STD grTexDownloadMipMapLevelPartial(	GrChipID_t tmu, FxU32 startAddress,
													GrLOD_t thisLod, GrLOD_t largeLod,
													GrAspectRatio_t   aspectRatio,
													GrTextureFormat_t format,
													FxU32 evenOdd, void *data,
													int start, int end )
{
}

// Transfer an NCC table or color palette.
EXPORT void STD grTexDownloadTable( GrChipID_t tmu, GrTexTable_t type, void *data )
{
	int i;
	switch( type )
	{
		// Only support this type of texture, for now.
		case GR_TEXTABLE_PALETTE:
			pDTDState->bPalDirty = true;
			// Transfer and translate the palette data.
			for( i=0; i<256; i++ )
			{
				pDTDState->pTexPalEntries[i].peRed	= (BYTE)(((DWORD*)data)[i]>>16) & 0x000000FF;
				pDTDState->pTexPalEntries[i].peGreen= (BYTE)(((DWORD*)data)[i]>>8) & 0x000000FF;
				pDTDState->pTexPalEntries[i].peBlue	= (BYTE)(((DWORD*)data)[i] & 0x000000FF);
			}
			break;

		default:
			DebugOut( "NCC tables are unsupported.\n" );
			break;
	}

	// Announce the texture settings are dirty.
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kTexture;
}

// Transfer part of an NCC table or color palette.
EXPORT void STD grTexDownloadTablePartial( GrChipID_t tmu, GrTexTable_t type, void *data, int start, int end )
{
	int i;
	switch( type )
	{
		// Only support this type of texture, for now.
		case GR_TEXTABLE_PALETTE:
			pDTDState->bPalDirty = true;
			// Transfer and translate the palette data.
			for( i=start; i<=end; i++ )
			{
				pDTDState->pTexPalEntries[i].peRed	= (BYTE)(((DWORD*)data)[i-start]>>16) & 0x000000FF;
				pDTDState->pTexPalEntries[i].peGreen= (BYTE)(((DWORD*)data)[i-start]>>8) & 0x000000FF;
				pDTDState->pTexPalEntries[i].peBlue	= (BYTE)(((DWORD*)data)[i-start] & 0x000000FF);
			}
			break;

		default:
			DebugOut( "NCC tables are unsupported.\n" );
			break;
	}

	// Announce the texture settings are dirty.
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kTexture;
}

// Set the minification and magnification filering method.
EXPORT void STD grTexFilterMode(	GrChipID_t tmu,
								GrTextureFilterMode_t minFilterMode,
								GrTextureFilterMode_t magFilterMode )
{
}

// Set the LOD biasing value.
EXPORT void STD grTexLodBiasValue( GrChipID_t tmu, float bias )
{
}

// Return the lowest address for texture space.
EXPORT FxU32 STD grTexMinAddress( GrChipID_t tmu )
{
	return 0;
}

// Return the highest address for texture space.
EXPORT FxU32 STD grTexMaxAddress( GrChipID_t tmu )
{
//	return 2048*1024;                   // Two megs of texture space.
	return pGlideState->nMaxTextureMemory;
}

// Set the mipmapping mode.
EXPORT void STD grTexMipMapMode( GrChipID_t tmu, GrMipMapMode_t mode, FxBool lodBlend )
{
}

// Set the multibase addressing mode. (Toss all textures from the Track & Trash manager)
EXPORT void STD grTexMultibase( GrChipID_t tmu, FxBool enable )
{
}

// Enables the address location for a multibased texture. (Create new texture)
EXPORT void STD grTexMultibaseAddress(	GrChipID_t tmu, GrTexBaseRange_t range,
									FxU32 startAddress, FxU32 evenOdd, GrTexInfo *info )
{
}


// Pick an NCC table.  (Watch out here, 3Dfx has a patent on this NCC stuff, don't support it.)
EXPORT void STD grTexNCCTable( GrChipID_t tmu, GrNCCTable_t table )
{
}

// Set the texture source for rendering.
EXPORT void STD grTexSource( GrChipID_t tmu, FxU32 startAddress, FxU32 evenOdd, GrTexInfo *pInfo )
{
	// Remember the current texture.
	pGlideState->DirtyFlags = GlideState::EDirtyFlags::kTexture;
	pGlideState->tex_address = startAddress;

	VTexture* pTmp = pDTDState->GetTextureAt(startAddress);
	if( pTmp ) // Only update the current texture if the start address is really valid.
	{
		pDTDState->pCurrentTexture = pTmp;
	} else
	{	// Failed to find a valid texture, so define a new one.

		// Make sure we're not in a scene drawing state.
		// It could be bad if the texture class performs a surface lock, and if the 3D card doesn't
		// allow that when in the middle of a scene.
		pDTDState->EndScene();

		// Create a new texture, using the given info.
		DWORD nTexSize = grTexTextureMemRequired( evenOdd, pInfo );
		pDTDState->pCurrentTexture = pDTDState->CreateAt( startAddress, startAddress+nTexSize-1, pInfo );

		// Setup the new texture's info.
		pDTDState->pCurrentTexture->Update();
	}

	// Announce the texture settings are dirty.
	pGlideState->DirtyFlags |= GlideState::EDirtyFlags::kTexture;
}

// Return the space occupied by a texture.
EXPORT FxU32 STD grTexTextureMemRequired( FxU32 evenOdd, GrTexInfo *info )
{
	if( !info)
		return 0;

	return grTexCalcMemRequired(  info->smallLod, info->largeLod, info->aspectRatio, info->format);
}

// TBI - do as it says.. figure out the RLE format.
EXPORT void STD ConvertAndDownloadRle( GrChipID_t tmu,
                        FxU32             startAddress,
                        GrLOD_t           thisLod,
                        GrLOD_t           largeLod,
                        GrAspectRatio_t   aspectRatio,
                        GrTextureFormat_t format,
                        FxU32             evenOdd,
                        FxU8              *bm_data,
                        long              bm_h,
                        FxU32             u0,
                        FxU32             v0,
                        FxU32             width,
                        FxU32             height,
                        FxU32             dest_width,
                        FxU32             dest_height,
                        FxU16             *tlut)
{
}



//-------------------------------------------------------------------
// Utility functions used by the texture routines.

//-----------------------------------
// DESCRIPTION: Prototype in GlideState.h
//		Calculates the size of the texture given the following inputs
//		INPUT
//			GrLod_t						Lod
//			GrAspectRatio_t		aspectRatio
//			GrTextureFormat_t format
//		OUTPUT
//			int nWidth
//			int nHeight
//			int nByte
//		See GrTexInfo for more information
//
//		nWidth and nHeight returns the width and height of the texture
//		nByte returns the number of bytes per pixel
//
// AUTHOR:
//    Modified by Khalid Shaikh (kshaikh@ece.mcgill.ca) on 01/09/99
//    Revised by Aaron Hilton (VideoGamer@home.com) on 02/14/99
//-----------------------------------
void GetTexInfo(const int Lod, const int aspectRatio, const int format, int &nWidth, int &nHeight, int &nByte)
{
	int nLength;

	switch(Lod)
	{
		case GR_LOD_256:  nLength = 256;  break;
		case GR_LOD_128:  nLength = 128;  break;
		case GR_LOD_64:   nLength = 64;   break;
		case GR_LOD_32:   nLength = 32;   break;
		case GR_LOD_16:   nLength = 16;   break;
		case GR_LOD_8:    nLength = 8;    break;
		case GR_LOD_4:    nLength = 4;    break;
		case GR_LOD_2:    nLength = 2;    break;
		case GR_LOD_1:    nLength = 1;    break;

	default:
		DebugOut("GetTextInfo() - Invalid largeLod: %d\n", Lod);
		return;
	}

	switch(aspectRatio)
	{
		case GR_ASPECT_8x1: nWidth = nLength;   nHeight = nLength/8;break;
		case GR_ASPECT_4x1: nWidth = nLength;   nHeight = nLength/4;break;
		case GR_ASPECT_2x1: nWidth = nLength;   nHeight = nLength/2;break;
		case GR_ASPECT_1x1: nWidth = nLength;   nHeight = nLength;  break;
		case GR_ASPECT_1x2: nWidth = nLength/2; nHeight = nLength;  break;
		case GR_ASPECT_1x4: nWidth = nLength/4; nHeight = nLength;  break;
		case GR_ASPECT_1x8: nWidth = nLength/8; nHeight = nLength;	break;

	default:
		DebugOut("GetTextInfo() - Invalid aspectRatio: %d\n", aspectRatio);
		return;
	}

	switch(format)
	{
		case GR_TEXFMT_RGB_332:
		case GR_TEXFMT_YIQ_422:
		case GR_TEXFMT_ALPHA_8:
		case GR_TEXFMT_INTENSITY_8:
		case GR_TEXFMT_ALPHA_INTENSITY_44:
		case GR_TEXFMT_P_8:
			nByte = 1;
			break;

		case GR_TEXFMT_RGB_565:
		case GR_TEXFMT_ARGB_8332:
		case GR_TEXFMT_AYIQ_8422:
		case GR_TEXFMT_ARGB_1555:
		case GR_TEXFMT_ARGB_4444:
		case GR_TEXFMT_ALPHA_INTENSITY_88:
		case GR_TEXFMT_AP_88:
			nByte = 2;
			break;

		default:
		  DebugOut("GetTextInfo() - Invalid format: %d\n", format);
		  return;
	}
}
