// DTDState : Contains the active state of Direct3D.
//

#if !defined(__DTDState_H__)
#define __DTDState_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class DTDState
{
	public:
	DTDState();
	~DTDState();

	// Prepare the texturing configuration for whatever the video hardware is capable of using.
	void PrepareTextureModes();

	// Destroy all internal objects.
	void Toast();

	// Scene control functions, useful for making sure
	// the BeginScene and EndScene pairs are always
	// correctly synched.
	void BeginScene();	// Makes sure the Direct3D drawing features are in BeginScene state.
	void EndScene();	// Makes sure the Direct3D drawing features are in EndScene state.
	bool bSceneReady;


	// Direct3D interpreted modes.

	// Color usage mode.
	enum EMode {
		kIterated = 0,
		kConstant,
		kNone
	};

	EMode	eColorMode; // Determines how the colors are being used in Direct3D.
	DWORD	dwConstantColor; // Translated constant Direct3D color.
	DWORD	dwConstantColorNoAlpha; // Translated constant Direct3D color.

	EMode	eAlphaMode; // Determines how the alpha values are being used in Direct3D.
	DWORD	dwConstantAlpha; // Translated constant Direct3D alpha.

	// Chroma keying mode.
	DWORD	dwChromaKeyColor;

	// Fog settings.
	DWORD dwFogColor;

	// ZBiasing level, precomputed to Direct3D.
	float fZBias;

	// Texture info.
	bool bUseTextures;
	bool bUseBlending;

	float fTexWidth;
	float fTexHeight;

	// Use this linked list, pointer array, and methods to Tag and Trash textures.
	// TBI - Re-evaluate the pointer array method, as it may be more efficient to just walk through the linked list.
	// note: I've already yoinked the pointer array.  It's a pain to implement efficiently.
	VTexture* CreateAt( DWORD iStart, DWORD iEnd, GrTexInfo *pInfo );
	VTexture* GetTextureAt( DWORD addressStart );
	void TrashTextureRange( DWORD iStart, DWORD iEnd );

	VTexture *pFirstTexture;
	VTexture *pCurrentTexture;

	// This is a buffer containing texture pointers. It the
	// length of GlideState's texture memory divided by eight.
	VTexture **pTextureBuffer;

	// Fills in the given texture range.
	void SetTextureRange( VTexture *pTexture, DWORD iStart );

	PALETTEENTRY pTexPalEntries[256];
	bool bPalDirty; // Determines if the texture palette is dirty,
					// if so and we have a palettized texture, then we need to update its palette.

	// Vertex translation and scaling (X and Y axis only) to
	// correct for various viewport resolutions.
	float fTranslateX, fTranslateY;
	float fScaleX, fScaleY;

	// Y Origin flipping.
	bool	bFlipY;
	float	fViewportWidth, fViewportHeight;

	// Direct3D framework.
	CD3DFramework	Framework;
	HWND			hWnd;		// D3D window handle.
	HWND			hParentWnd; // D3D parent window handle.
};

extern DTDState* pDTDState;

//-------------------------------------------------------------------
// Color conversion macro functions.  Located in DGColor.cpp
//-------------------------------------------------------------------

// Convert to 8888 color format (Direct3D colors)
DWORD BGRA8888toRGBA5650( DWORD &dwBGRA8888 );
DWORD BGRA8888toARGB8888( DWORD &dwBGRA8888 );
DWORD RGBA8888toRGBA5650( DWORD &dwRGBA8888 );
DWORD RGBA8888toARGB8888( DWORD &dwRGBA8888 );

// Convert to 5650 color format (SurfaceDIB colors)
DWORD ABGR8888toRGBA5650( DWORD &dwABGR8888 );
DWORD ABGR8888toARGB8888( DWORD &dwABGR8888 );
DWORD ARGB8888toRGBA5650( DWORD &dwARGB8888 );
DWORD ARGB8888toARGB8888( DWORD &dwARGB8888 );

// Indirection pointer functions.
extern DWORD (*DGColorARGB8888)( DWORD &dw8888 );
extern DWORD (*DGColorARGB5650)( DWORD &dw8888 );

//-------------------------------------------------------------------
// Texture internal utility functions.
//-----------------------------------
// Convert the LOD info to an integer.
int ConvLodToInt(GrLOD_t lod);

// Convert the integer to an LOD state value.
GrLOD_t ConvIntToLod(int nLod);

// Convert the aspect ratio into Width and Height info.
void ConvAspectToInt(int &nAspectW, int &nAspectH, GrAspectRatio_t aspect_ratio);

// Convert a Width and Height into an aspect ratio state value.
void ConvIntToAspect( GrAspectRatio_t &aspect_ratio, int nAspectW, int nAspectH);


//-------------------------------------------------------------------
// VTexture classes.
// Add your new texture support classes in here. - Aaron (VideoGamer@home.com)
#include "DTDTexture_P_8.h"
#include "DTDTexture_RGB_565_EmulateP8.h"
#include "DTDTexture_ARGB_4444.h"
#include "DTDTexture_AP_88.h"
#include "DTDTexture_ARGB_4444_EmulateAP88.h"

//-------------------------------------------------------------------
// TBI - Anyone care to implement any of these generic texture modes?
// #include "DTDTexture_GenericHAL.h"
// #include "DTDTexture_GenericHEL.h"
// #include "DTDTexture_GenericHEL_P_8.h"
// #include "DTDTexture_GenericHEL_AP_8.h"
// #include "DTDTexture_GenericHEL_ARGB_4444.h"
// #include "DTDTexture_GenericHEL_RGB_565.h"


#endif // __DTDState_H__






















