//-------------------------------------------------------------------
//	File:		DTDTexture_ARGB_4444.cpp
//	Created:	17/02/99 0:19AM
//	Author:		Aaron Hilton
//	Comments:	Implement the combined ARGB 4444bit texture format.
//-------------------------------------------------------------------
#include "stdafx.h"
#include "DTDTexture_ARGB_4444.h"

// TBI - Must implement proper MipMap support.

// Create the texture from a given (inclusive) address range.
// Refer to pGlideState->pTexMemory for the texture memory.
bool DTDTexture_ARGB_4444::Create()
{
	// Relocate our m_Info's data pointer to the start of our texture frame buffer data.
	m_Info.data = &pGlideState->pTexMemory[m_iBegin];

	// Lay down the surface information. (Thanks go out to Let)
	DDSURFACEDESC2 surfDesc;
	ZeroMemory(&surfDesc, sizeof(DDSURFACEDESC2));
	surfDesc.dwSize = sizeof(DDSURFACEDESC2);	// Fill the structure's size info.
	surfDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;// | DDSD_MIPMAPCOUNT;	// Say we're going to use this info.
	surfDesc.ddsCaps.dwCaps = DDSCAPS_TEXTURE;
	surfDesc.ddsCaps.dwCaps2 = DDSCAPS2_TEXTUREMANAGE;	// Make this a self managed texture.

	// TBI - Prepare the real mipmap setting.
	surfDesc.dwMipMapCount = 1;

	// Prepare the texture format info.
	surfDesc.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	surfDesc.dwWidth = nWidth;
	surfDesc.dwHeight = nHeight;

	// Configure the surface description for an alpha embeded RGB texture.
	DWORD nFormatWidth = nWidth*2;
	surfDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_ALPHAPIXELS;
	surfDesc.ddpfPixelFormat.dwRGBBitCount = 16;
	surfDesc.ddpfPixelFormat.dwRBitMask = 0x00000F00;
	surfDesc.ddpfPixelFormat.dwGBitMask = 0x000000F0;
	surfDesc.ddpfPixelFormat.dwBBitMask = 0x0000000F;
	surfDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0x0000F000;

	// We do not use a palette.
	bPal = false;

	// Attempt to create the texture's surface.
	if( pDTDState->Framework.GetDirectDraw()->CreateSurface(&surfDesc, &pSurface, 0 ) != DD_OK )
	{ // Failed to create the texture, so leave it at that.
		pSurface = 0;
		DebugOut( "DTDTexture_ARGB_4444: Failed to create a texture surface.\n" );
		return false;
	}

	// Grab the texture interface.
	pSurface->QueryInterface(IID_IDirect3DTexture2, (LPVOID*)&pTexture);

	return true;
}

// Texture needs updating, so fill it in with the texture buffer.
void DTDTexture_ARGB_4444::Update()
{
	// Make sure we have a texture surface before updating.
	Assert( pSurface );

	// Make sure we're outside of a scene state.
	pDTDState->EndScene();

	// Handy variable, used by many other DTDTexture formats, helps with generalizing the code.
	DWORD nFormatWidth = nWidth*2;        // 16bpp textures have a double sized format width.

	// We have everything at this point, so let's transfer the relevent texture info.
	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	pSurface->GetSurfaceDesc(&ddsd);
	if( pSurface->Lock(NULL, &ddsd, DDLOCK_WRITEONLY | DDLOCK_WAIT, NULL) != DD_OK )
	{ // Couldn't lock the texture's surface for the blitting, should be interesting. (Random data!)
		DebugOut( "DTDTexture_ARGB_4444: Failed to lock the texture's surface for blitting.\n" );
		return;
	}

	// Blit the whole source texture data into the target surface.
	Assert( nHeight == (int)ddsd.dwHeight );
	for( int y=0; y<nHeight; y++ )
	{
		// Copy a scan line at a time.
		memcpy( (BYTE*)ddsd.lpSurface + (y*ddsd.lPitch), (BYTE*)m_Info.data + (y*nFormatWidth), nFormatWidth );
	}

	// Unlock the surface
	pSurface->Unlock(NULL);
}


//-------------------------------------------------------------------
//	History:
//	$Log:$
//-------------------------------------------------------------------
