//-------------------------------------------------------------------
//	File:		DTDTexture_P_8.cpp
//	Created:	16/02/99 11:59PM
//	Author:		Aaron Hilton
//	Comments:	Implement the 8bit palettized texture format.
//-------------------------------------------------------------------
#include "stdafx.h"
#include "DTDTexture_P_8.h"

// TBI - Must implement proper MipMap support.

// Create the texture from a given (inclusive) address range.
// Refer to pGlideState->pTexMemory for the texture memory.
bool DTDTexture_P_8::Create()
{
	// Relocate our m_Info's data pointer to the start of our texture frame buffer data.
	m_Info.data = &pGlideState->pTexMemory[m_iBegin];

	// Lay down the surface information. (Thanks go out to Let)
	DDSURFACEDESC2 surfDesc;
	ZeroMemory(&surfDesc, sizeof(DDSURFACEDESC2));
	surfDesc.dwSize = sizeof(DDSURFACEDESC2);	// Fill the structure's size info.
	surfDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;// | DDSD_MIPMAPCOUNT;	// Say we're going to use this info.
	surfDesc.ddsCaps.dwCaps = DDSCAPS_TEXTURE;
	surfDesc.ddsCaps.dwCaps2 = DDSCAPS2_TEXTUREMANAGE;	// Make this a self managed texture.

	// TBI - Prepare the real mipmap setting.
	surfDesc.dwMipMapCount = 1;

	// Prepare the texture format info.
	surfDesc.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	surfDesc.dwWidth = nWidth;
	surfDesc.dwHeight = nHeight;

	// Configure the surface description for a palettized texture.
	DWORD nFormatWidth = nWidth;
	surfDesc.ddpfPixelFormat.dwFlags = DDPF_RGB | DDPF_PALETTEINDEXED8;
	surfDesc.ddpfPixelFormat.dwRGBBitCount = 8;
	surfDesc.ddpfPixelFormat.dwRBitMask = 0x00000000;
	surfDesc.ddpfPixelFormat.dwGBitMask = 0x00000000;
	surfDesc.ddpfPixelFormat.dwBBitMask = 0x00000000;
	surfDesc.ddpfPixelFormat.dwRGBAlphaBitMask = 0x00000000;

	// Yes, We do use a palette.
	bPal = true;

	// Attempt to create the texture's surface.
	if( pDTDState->Framework.GetDirectDraw()->CreateSurface(&surfDesc, &pSurface, 0 ) != DD_OK )
	{ // Failed to create the texture, so leave it at that.
		pSurface = 0;
		DebugOut( "DTDTexture_P_8: Failed to create a texture surface.\n" );
		return false;
	}

	// Grab the texture interface.
	pSurface->QueryInterface(IID_IDirect3DTexture2, (LPVOID*)&pTexture);

	// Create the palette.
	if( pDTDState->Framework.GetDirectDraw()->CreatePalette(DDPCAPS_8BIT, pDTDState->pTexPalEntries, &pPalette, NULL) != DD_OK )
	{ // Failed to create the palette, so leave it and go on without one. Hehehe.. should look strange.
		pPalette = 0;
		bPal = false;
		DebugOut( "DTDTexture_P_8: Failed to create a palette, for a palettized texture.\n" );
		return false;
	}

	// Associate it with our texture surface.
	pSurface->SetPalette( pPalette );
	return true;
}

// Texture needs updating, so fill it in with the texture buffer.
void DTDTexture_P_8::Update()
{
	// Make sure we have a texture surface before updating.
	Assert( pSurface );

	// Make sure we're outside of a scene state.
	pDTDState->EndScene();

	// Handy variable, used by many other DTDTexture formats, helps with generalizing the code.
	DWORD nFormatWidth = nWidth;        // 8bpp textures have an equal format width, to their nWidth.

	// We have everything at this point, so let's transfer the relevent texture info.
	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	pSurface->GetSurfaceDesc(&ddsd);
	if( pSurface->Lock(NULL, &ddsd, DDLOCK_WRITEONLY | DDLOCK_WAIT, NULL) != DD_OK )
	{ // Couldn't lock the texture's surface for the blitting, should be interesting. (Random data!)
		DebugOut( "DTDTexture_P_8: Failed to lock the texture's surface for blitting.\n" );
		return;
	}

	// Blit the whole source texture data into the target surface.
	Assert( nHeight == (int)ddsd.dwHeight );
	for( int y=0; y<nHeight; y++ )
	{
		// Copy a scan line at a time.
		memcpy( (BYTE*)ddsd.lpSurface + (y*ddsd.lPitch), (BYTE*)m_Info.data + (y*nFormatWidth), nFormatWidth );
	}

	// Unlock the surface
	pSurface->Unlock(NULL);
}

// The palette needs updating.
// (Palette buffer is dirty, and we're about to draw!)
void DTDTexture_P_8::UpdatePal()
{
	pPalette->SetEntries( 0, 0, 256, pDTDState->pTexPalEntries );
}


//-------------------------------------------------------------------
//	History:
//	$Log:$
//-------------------------------------------------------------------
