//-------------------------------------------------------------------
// Debug - Debug system, and information.
// Standard debugging features.
//-------------------------------------------------------------------
#ifndef __DEBUG_H_
#define __DEBUG_H_


//-------------------------------------------
//		Types and defines
//-------------------------------------------
// Change this to increase the allowed length of the debug string.
const int kMaxDebugMessageLength = 1024;

// Validate the expression is TRUE, otherwise call the debug notification function, VerifyMessage.
#define Verify(expression, errcode) if (!(expression)) ::VerifyMessage(errcode, #expression, __FILE__, __LINE__)

// Assert the expression, and aborts execution only within a debug build.
#ifdef __DEBUG
	#define Assert(expression) ((expression) ? (void)0 \
		: VerifyMessage(0, #expression, __FILE__, __LINE__))
#else
	#define Assert(test) (void)0
#endif

//-------------------------------------------
// Generic debug checkpoint code.
extern unsigned long g_debugCode;

// Check points are useful for tracking the point of our crash.
#ifdef __DEBUG
	extern const char* g_debugString;
	extern const char* g_debugFile;
	extern long g_debugLineNumber;
	inline void CheckPoint(unsigned long nCode, const char* szString)
	{
		g_debugCode = nCode;
		g_debugString = szString;
		g_debugFile = __FILE__;
		g_debugLineNumber = __LINE__;
	}
#else
	inline void CheckPoint(unsigned long nCode, const char*)
	{
		g_debugCode = nCode;
	}
#endif

//-------------------------------------------
//		Global Prototypes
//-------------------------------------------
// print a formatted text string to debug device
#ifdef __DEBUG
	void DebugOut(const char* szFormat, ...);
#else
	inline void DebugOut(const char*, ...) {}
#endif

// Internal VerifyMsg function.  Not to be used by anything but Debug.h and .cpp
void VerifyMessage(long errStrIndex, const char* exprString, const char* file, long linenumber);

#endif // __PDEBUG_H_
