// GlideState : Contains the active state of Glide.
//

#if !defined(__GlideState_H__)
#define __GlideState_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//-------------------------------------------------------------------
// Name: GetTexInfo() - in DGTex.cpp
// Desc: Calculates the size of the texture given the following inputs.
void GetTexInfo(const int Lod, const int aspectRatio, const int format, int &nWidth, int &nHeight, int &nByte);
//-------------------------------------------------------------------

//-------------------------------------------------------------------
// Name: ConvertScreenRes()
// Desc: Utility function - fills the dwHeight and dwWidth variables
//		 with the screen resolution specified by "res".
//-----------------------------------------------------------------------------
void ConvertScreenRes( GrScreenResolution_t res, DWORD &dwWidth, DWORD &dwHeight );

//-----------------------------------------------------------------------------
// Name: ConvertRefreshRate()
// Desc: Utility function - returns the translated refresh rate.
//-----------------------------------------------------------------------------
DWORD ConvertRefreshRate( GrScreenRefresh_t refresh );

class GlideState
{
	public:
	GlideState();
	~GlideState();

	// Flush the states set by the DirtyFlags.
	void FlushStates();
	void GetState( GrState *state );
	void SetState( const GrState *state );

	// Maintain a list of all unique blending states.
	UniqueBlendState *pUniqueState;
	FILE *fileUniqueStates;

	// Glide Sst window parameters.
	GrScreenResolution_t	screen_resolution;
	GrColorFormat_t			color_format;
	GrOriginLocation_t		origin_location;
	int nColBuffers;
	int nAuxBuffers;

	// Alpha blending.
	GrAlphaBlendFnc_t alpha_rgb_source;
	GrAlphaBlendFnc_t alpha_rgb_dest;
	GrAlphaBlendFnc_t alpha_alpha_source;
	GrAlphaBlendFnc_t alpha_alpha_dest;

	// Alpha combine processing.
	GrCombineFunction_t	alpha_func;
	GrCombineFactor_t	alpha_factor;
	GrCombineLocal_t	alpha_local;
	GrCombineOther_t	alpha_other;
	FxBool				alpha_invert;

	// High level alpha combine mode.
	GrAlphaSource_t alpha_mode;

	// Hi bit iterated RGB lighting.  FX = Self Lit texture pixels.
	FxBool alpha_ITRGBLighting;

	// Chromakey, in the alpha stage.
	GrCmpFnc_t	alpha_test_func;
	GrAlpha_t	alpha_test_value;

	// Chromakey, in the pixelFX stage.
	GrChromakeyMode_t	chroma_mode;
	GrColor_t			chroma_value;

	// Clip Window.
	FxU32 clip_minx, clip_miny;
	FxU32 clip_maxx, clip_maxy;

	// Color combine processing.
	GrCombineFunction_t	color_func;
	GrCombineFactor_t	color_factor;
	GrCombineLocal_t	color_local;
	GrCombineOther_t	color_other;
	FxBool				color_invert;

	// High level color combine function.
	GrColorCombineFnc_t color_combine_func;

	// Write masking enable/disable. (ColorMask, and AlphaMask)
	FxBool mask_rgb, mask_alpha;

	// Constant color value.
	GrColor_t color_value;

	// Backface culling.
	GrCullMode_t cull_mode;

	// Depth buffer control.
	FxI16				depth_bias;
	GrCmpFnc_t			depth_func;
	GrDepthBufferMode_t	depth_mode;
	FxBool				depth_mask;

	// Dither mode.
	GrDitherMode_t dither_mode;

	// Fog control.
	GrColor_t	fog_value;
	GrFogMode_t	fog_mode;
	GrFog_t		fog_table[GR_FOG_TABLE_SIZE];

	// Gamma correction value.
	float gamma_value;

	// Linear framebuffer control.
	GrAlpha_t	lfb_alpha;
	FxU16		lfb_depth;

	// Rendering target buffer.
	GrBuffer_t render_buffer;

	// Origin info, UpperLeft (going down) or LowerLeft (going up).
	GrOriginLocation_t origin;

	// Texture clamping.
	GrTextureClampMode_t tex_sClampMode;
	GrTextureClampMode_t tex_tClampMode;

	// Texture combine.
	GrCombineFunction_t	tex_rgb_func;
	GrCombineFactor_t	tex_rgb_factor;
	GrCombineFunction_t	tex_alpha_func;
	GrCombineFactor_t	tex_alpha_factor;
	FxBool				tex_rgb_invert;
	FxBool				tex_alpha_invert;

	// High level texture combine function.
	GrTextureCombineFnc_t tex_combine_func;

	// Texture filtering mode.
	GrTextureFilterMode_t tex_minFilterMode;
	GrTextureFilterMode_t tex_magFilterMode;

	// LOD biasing control.
	float tex_LODbias;

	// Texture mipmapping mode.
	GrMipMapMode_t	tex_mipmap_mode;
	FxBool			tex_mipmap_lodBlend;

	// Texture selection info.
	FxU32 tex_address;
	FxU32 tex_evenOdd;

	// High level texture selection info.
	GrMipMapId_t tex_select;

	// Represented texture dimensions.
	float fTextureWidth;
	float fTextureHeight;

	// NCC table control. (AKA. palette selection)
//	GrNCCTable_t	ncc_table;

	// Texture memory buffer.
	DWORD nMaxTextureMemory;  // TBI - Verify the code always checks against this value, to prevent buffer overrun.
	BYTE *pTexMemory;

	// Handy flags to determine what state changed since last drawn triangle.
	// TBI - we need to encapsulate the flag setting opperations!
	unsigned long DirtyFlags;
	enum EDirtyFlags
	{
		kAlpha	= 1,
		kColor	= 1<<1,
		kMask	= 1<<2,
		kDepth	= 1<<3,
		kCull	= 1<<4,
		kClip	= 1<<5,
		kTexture		= 1<<6,
		kTextureMipmap	= 1<<7,
		kTextureSize	= 1<<8
	};
};

extern GlideState *pGlideState;

#endif // __GlideState_H__
