//-------------------------------------------------------------------
//	File:		UniqueBlendState.cpp
//	Created:	25/02/99 2:04AM
//	Author:		Aaron Hilton
//	Comments:
//-------------------------------------------------------------------
#include "stdafx.h"
#include "UniqueBlendState.h"

char* ConvertAlphaBlend( GrAlphaBlendFnc_t val );

char* ConvertFunction( GrCombineFunction_t val);
char* ConvertFactor( GrCombineFactor_t val );
char* ConvertLocal( GrCombineLocal_t val );
char* ConvertOther( GrCombineOther_t val );

char* ConvertCompareFunction( GrCmpFnc_t val );
char* ConvertColorCombineFunction( GrColorCombineFnc_t val );
char* ConvertTextureCombineFunction( GrTextureCombineFnc_t val );


UniqueBlendState::UniqueBlendState()
{
	pNext = 0;

	alpha_rgb_source = pGlideState->alpha_rgb_source;
	alpha_rgb_dest = pGlideState->alpha_rgb_dest;
	alpha_alpha_source = pGlideState->alpha_alpha_source;
	alpha_alpha_dest = pGlideState->alpha_alpha_dest;

	alpha_func   = pGlideState->alpha_func;
	alpha_factor = pGlideState->alpha_factor;
	alpha_local  = pGlideState->alpha_local;
	alpha_other  = pGlideState->alpha_other;
	alpha_invert = pGlideState->alpha_invert;

	alpha_test_func  = pGlideState->alpha_test_func;
	alpha_test_value = pGlideState->alpha_test_value;

	chroma_mode  = pGlideState->chroma_mode;
	chroma_value = pGlideState->chroma_value;

	color_func   = pGlideState->color_func;
	color_factor = pGlideState->color_factor;
	color_local  = pGlideState->color_local;
	color_other  = pGlideState->color_other;
	color_invert = pGlideState->color_invert;

	color_value = pGlideState->color_value;
}

// Identify if all of these are equal.
bool UniqueBlendState::IsEqual( UniqueBlendState* pState )
{
	bool bEqual = true; // If any of these are false, then bEqual propagates a "false" all the way down to the return.
	bEqual &= (alpha_rgb_source == pState->alpha_rgb_source);
	bEqual &= (alpha_rgb_dest == pState->alpha_rgb_dest);
	bEqual &= (alpha_alpha_source == pState->alpha_alpha_source);
	bEqual &= (alpha_alpha_dest == pState->alpha_alpha_dest);

	bEqual &= (alpha_func   == pState->alpha_func);
	bEqual &= (alpha_factor == pState->alpha_factor);
	bEqual &= (alpha_local  == pState->alpha_local);
	bEqual &= (alpha_other  == pState->alpha_other);
	bEqual &= (alpha_invert == pState->alpha_invert);

	bEqual &= (alpha_test_func  == pState->alpha_test_func);
	bEqual &= (alpha_test_value == pState->alpha_test_value);

	bEqual &= (chroma_mode  == pState->chroma_mode);
	bEqual &= (chroma_value == pState->chroma_value);

	bEqual &= (color_func   == pState->color_func);
	bEqual &= (color_factor == pState->color_factor);
	bEqual &= (color_local  == pState->color_local);
	bEqual &= (color_other  == pState->color_other);
	bEqual &= (color_invert == pState->color_invert);

//	bEqual &= (color_value == pState->color_value);

	return bEqual;
}

void UniqueBlendState::Print( FILE *f )
{
	UniqueBlendState *pPrevState = pNext;
	if( !pPrevState )
		pPrevState = this;

	static int iState = 0;
	iState++;

	fprintf(f,	"-------------------------------------------------------------------\n"
				" Unique Blend State: %d\n"
				"---------------------\n\n", iState);

	if( pPrevState->alpha_rgb_source != alpha_rgb_source ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_rgb_source (%d): %s\n",	alpha_rgb_source,	ConvertAlphaBlend(alpha_rgb_source) );
	if( pPrevState->alpha_rgb_dest != alpha_rgb_dest ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_rgb_dest (%d): %s\n",	alpha_rgb_dest,		ConvertAlphaBlend(alpha_rgb_dest) );
	if( pPrevState->alpha_alpha_source != alpha_alpha_source ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_alpha_source (%d): %s\n",alpha_alpha_source,	ConvertAlphaBlend(alpha_alpha_source) );
	if( pPrevState->alpha_alpha_dest != alpha_alpha_dest ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_alpha_dest (%d): %s\n\n",alpha_alpha_dest,	ConvertAlphaBlend(alpha_alpha_dest) );

	if( pPrevState->alpha_func != alpha_func ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_func (%d): %s\n",		alpha_func,		ConvertFunction(alpha_func) );
	if( pPrevState->alpha_factor != alpha_factor ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_factor (%d): %s\n",		alpha_factor,	ConvertFactor(alpha_factor) );
	if( pPrevState->alpha_local != alpha_local ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_local (%d): %s\n",		alpha_local,	ConvertLocal(alpha_local) );
	if( pPrevState->alpha_other != alpha_other ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_other (%d): %s\n",		alpha_other,	ConvertOther(alpha_other) );
	if( pPrevState->alpha_invert != alpha_invert ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_invert: %s\n\n",			alpha_invert?"true":"false" );

	if( pPrevState->alpha_test_func != alpha_test_func ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_test_func (%d): %s\n",	alpha_test_func, ConvertCompareFunction(alpha_test_func) );
	if( pPrevState->alpha_test_value != alpha_test_value ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"alpha_test_value: %d\n\n",		(DWORD)alpha_test_value);

	if( pPrevState->chroma_mode != chroma_mode ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"chroma_mode (%d): %s\n",		chroma_mode, chroma_mode?"GR_CHROMAKEY_ENABLE":"GR_CHROMAKEY_DISABLE" );
	if( pPrevState->chroma_value != chroma_value ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"chroma_value: 0x%08X\n\n",		chroma_value);

	if( pPrevState->color_func != color_func ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"color_func (%d): %s\n",		color_func,		ConvertFunction(color_func) );
	if( pPrevState->color_factor != color_factor ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"color_factor (%d): %s\n",		color_factor,	ConvertFactor(color_factor) );
	if( pPrevState->color_local != color_local ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"color_local (%d): %s\n",		color_local,	ConvertLocal(color_local) );
	if( pPrevState->color_other != color_other ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"color_other (%d): %s\n",		color_other,	ConvertOther(color_other) );
	if( pPrevState->color_invert != color_invert ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"color_invert: %s\n\n",			color_invert?"true":"false" );

	if( pPrevState->color_value != color_value ) fprintf(f, "*\t" ); else fprintf(f, "\t" );
	fprintf(f,	"color_value: 0x%08X\n\n",		color_value);
}


//-----------------------------------
// Convert state info to strings.

char* ConvertAlphaBlend( GrAlphaBlendFnc_t val )
{
	switch( val )
	{
		case GR_BLEND_ZERO:					return "GR_BLEND_ZERO";
		case GR_BLEND_ONE:					return "GR_BLEND_ONE";
		case GR_BLEND_SRC_ALPHA:			return "GR_BLEND_SRC_ALPHA";
		case GR_BLEND_SRC_COLOR:			return "GR_BLEND_(SRC or DST)_COLOR";
		case GR_BLEND_DST_ALPHA:			return "GR_BLEND_DST_ALPHA";
		case GR_BLEND_ONE_MINUS_SRC_ALPHA:	return "GR_BLEND_ONE_MINUS_SRC_ALPHA";
		case GR_BLEND_ONE_MINUS_SRC_COLOR:	return "GR_BLEND_ONE_MINUS_(SRC or DST)_COLOR";
		case GR_BLEND_ONE_MINUS_DST_ALPHA:	return "GR_BLEND_ONE_MINUS_DST_ALPHA";
		case GR_BLEND_PREFOG_COLOR:			return "GR_BLEND_PREFOG_COLOR or GR_BLEND_ALPHA_SATURATE";
	}

	return "";
}

char* ConvertFunction( GrCombineFunction_t val)
{
	switch( val )
	{
		case GR_COMBINE_FUNCTION_LOCAL:										return "GR_COMBINE_FUNCTION_LOCAL";
		case GR_COMBINE_FUNCTION_LOCAL_ALPHA:								return "GR_COMBINE_FUNCTION_LOCAL_ALPHA";
		case GR_COMBINE_FUNCTION_SCALE_OTHER:								return "GR_COMBINE_FUNCTION_(SCALE or BLEND)_OTHER";
		case GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL:						return "GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL";
		case GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL_ALPHA:				return "GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL_ALPHA";
		case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL:					return "GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL";
//		case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL:			return "GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL";
		case GR_COMBINE_FUNCTION_BLEND:										return "GR_COMBINE_FUNCTION_BLEND or GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL";
		case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL_ALPHA:	return "GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL_ALPHA";
		case GR_COMBINE_FUNCTION_BLEND_LOCAL:								return "GR_COMBINE_FUNCTION_BLEND_LOCAL";
		case GR_COMBINE_FUNCTION_SCALE_MINUS_LOCAL_ADD_LOCAL_ALPHA:			return "GR_COMBINE_FUNCTION_SCALE_MINUS_LOCAL_ADD_LOCAL_ALPHA";
	}

	return "";
}

char* ConvertFactor( GrCombineFactor_t val )
{
	switch( val )
	{
		case GR_COMBINE_FACTOR_LOCAL:					return "GR_COMBINE_FACTOR_LOCAL";
		case GR_COMBINE_FACTOR_OTHER_ALPHA:				return "GR_COMBINE_FACTOR_OTHER_ALPHA";
		case GR_COMBINE_FACTOR_LOCAL_ALPHA:				return "GR_COMBINE_FACTOR_LOCAL_ALPHA";
		case GR_COMBINE_FACTOR_TEXTURE_ALPHA:			return "GR_COMBINE_FACTOR_TEXTURE_ALPHA or GR_COMBINE_FACTOR_DETAIL_FACTOR";
//		case GR_COMBINE_FACTOR_DETAIL_FACTOR:			return "GR_COMBINE_FACTOR_DETAIL_FACTOR";
		case GR_COMBINE_FACTOR_LOD_FRACTION:			return "GR_COMBINE_FACTOR_LOD_FRACTION";
		case GR_COMBINE_FACTOR_ONE:						return "GR_COMBINE_FACTOR_ONE";
		case GR_COMBINE_FACTOR_ONE_MINUS_LOCAL:			return "GR_COMBINE_FACTOR_ONE_MINUS_LOCAL";
		case GR_COMBINE_FACTOR_ONE_MINUS_OTHER_ALPHA:	return "GR_COMBINE_FACTOR_ONE_MINUS_OTHER_ALPHA";
		case GR_COMBINE_FACTOR_ONE_MINUS_LOCAL_ALPHA:	return "GR_COMBINE_FACTOR_ONE_MINUS_LOCAL_ALPHA";
		case GR_COMBINE_FACTOR_ONE_MINUS_TEXTURE_ALPHA:	return "GR_COMBINE_FACTOR_ONE_MINUS_TEXTURE_ALPHA or GR_COMBINE_FACTOR_ONE_MINUS_DETAIL_FACTOR";
//		case GR_COMBINE_FACTOR_ONE_MINUS_DETAIL_FACTOR:	return "GR_COMBINE_FACTOR_ONE_MINUS_DETAIL_FACTOR";
		case GR_COMBINE_FACTOR_ONE_MINUS_LOD_FRACTION:	return "GR_COMBINE_FACTOR_ONE_MINUS_LOD_FRACTION";
	}

	return "";
}

char* ConvertLocal( GrCombineLocal_t val )
{
	switch( val )
	{
		case GR_COMBINE_LOCAL_ITERATED:		return "GR_COMBINE_LOCAL_ITERATED";
		case GR_COMBINE_LOCAL_CONSTANT:		return "GR_COMBINE_LOCAL_CONSTANT";
		case GR_COMBINE_LOCAL_DEPTH:		return "GR_COMBINE_LOCAL_DEPTH";
	}

	return "";
}

char* ConvertOther( GrCombineOther_t val )
{
	switch( val )
	{
		case GR_COMBINE_OTHER_ITERATED:		return "GR_COMBINE_OTHER_ITERATED";
		case GR_COMBINE_OTHER_TEXTURE:		return "GR_COMBINE_OTHER_TEXTURE";
		case GR_COMBINE_OTHER_CONSTANT:		return "GR_COMBINE_OTHER_CONSTANT";
	}

	return "";
}

char* ConvertCompareFunction( GrCmpFnc_t val )
{
	switch ( val )
	{
		case GR_CMP_NEVER:		return "GR_CMP_NEVER";
		case GR_CMP_LESS:		return "GR_CMP_LESS";
		case GR_CMP_EQUAL:		return "GR_CMP_EQUAL";
		case GR_CMP_LEQUAL:		return "GR_CMP_LEQUAL";
		case GR_CMP_GREATER:	return "GR_CMP_GREATER";
		case GR_CMP_NOTEQUAL:	return "GR_CMP_NOTEQUAL";
		case GR_CMP_GEQUAL:		return "GR_CMP_GEQUAL";
		case GR_CMP_ALWAYS:		return "GR_CMP_ALWAYS";
	}

	return "";
}

char* ConvertColorCombineFunction( GrColorCombineFnc_t val )
{
	switch( val )
	{
		case GR_COLORCOMBINE_ZERO:		   					return "GR_COLORCOMBINE_ZERO";
		case GR_COLORCOMBINE_ONE:		   					return "GR_COLORCOMBINE_ONE";
		case GR_COLORCOMBINE_CCRGB:		   					return "GR_COLORCOMBINE_CCRGB";
		case GR_COLORCOMBINE_ITRGB:		   					return "GR_COLORCOMBINE_ITRGB";
		case GR_COLORCOMBINE_ITRGB_DELTA0: 					return "GR_COLORCOMBINE_ITRGB_DELTA0";
		case GR_COLORCOMBINE_DECAL_TEXTURE:					return "GR_COLORCOMBINE_DECAL_TEXTURE";
		case GR_COLORCOMBINE_TEXTURE_TIMES_CCRGB:			return "GR_COLORCOMBINE_TEXTURE_TIMES_CCRGB";
		case GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB:			return "GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB";
		case GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB_DELTA0:	return "GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB_DELTA0";
		case GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB_ADD_ALPHA:	return "GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB_ADD_ALPHA";
		case GR_COLORCOMBINE_TEXTURE_TIMES_ALPHA:			return "GR_COLORCOMBINE_TEXTURE_TIMES_ALPHA";
		case GR_COLORCOMBINE_TEXTURE_TIMES_ALPHA_ADD_ITRGB:	return "GR_COLORCOMBINE_TEXTURE_TIMES_ALPHA_ADD_ITRGB";
		case GR_COLORCOMBINE_TEXTURE_ADD_ITRGB:				return "GR_COLORCOMBINE_TEXTURE_ADD_ITRGB";
		case GR_COLORCOMBINE_TEXTURE_SUB_ITRGB:				return "GR_COLORCOMBINE_TEXTURE_SUB_ITRGB";
		case GR_COLORCOMBINE_CCRGB_BLEND_ITRGB_ON_TEXALPHA:	return "GR_COLORCOMBINE_CCRGB_BLEND_ITRGB_ON_TEXALPHA";
		case GR_COLORCOMBINE_DIFF_SPEC_A:					return "GR_COLORCOMBINE_DIFF_SPEC_A";
		case GR_COLORCOMBINE_DIFF_SPEC_B:					return "GR_COLORCOMBINE_DIFF_SPEC_B";
	}

	return "";
}

char* ConvertTextureCombineFunction( GrTextureCombineFnc_t val )
{
	switch( val )
	{
		case GR_TEXTURECOMBINE_ZERO:			return "GR_TEXTURECOMBINE_ZERO";
		case GR_TEXTURECOMBINE_ONE:				return "GR_TEXTURECOMBINE_ONE";
		case GR_TEXTURECOMBINE_DECAL:			return "GR_TEXTURECOMBINE_DECAL";
		case GR_TEXTURECOMBINE_OTHER:			return "GR_TEXTURECOMBINE_OTHER";
		case GR_TEXTURECOMBINE_ADD:				return "GR_TEXTURECOMBINE_ADD";
		case GR_TEXTURECOMBINE_MULTIPLY:		return "GR_TEXTURECOMBINE_MULTIPLY";
		case GR_TEXTURECOMBINE_SUBTRACT:		return "GR_TEXTURECOMBINE_SUBTRACT";
		case GR_TEXTURECOMBINE_DETAIL:			return "GR_TEXTURECOMBINE_DETAIL";
		case GR_TEXTURECOMBINE_DETAIL_OTHER:	return "GR_TEXTURECOMBINE_DETAIL_OTHER";
		case GR_TEXTURECOMBINE_TRILINEAR_ODD:	return "GR_TEXTURECOMBINE_TRILINEAR_ODD";
		case GR_TEXTURECOMBINE_TRILINEAR_EVEN:	return "GR_TEXTURECOMBINE_TRILINEAR_EVEN";
	}

	return "";
}

//-------------------------------------------------------------------
//	History:
//	$Log:$
//-------------------------------------------------------------------
