//-------------------------------------------------------------------
//	File:		VTexture.h
//	Created:	14/02/99 2:58AM
//	Author:		Aaron Hilton
//	Comments:	An interface class which provides all the base
//				functionality to make the texture management work.
//-------------------------------------------------------------------
#ifndef __VTexture_h_
#define __VTexture_h_

class VTexture
{
public:
	VTexture();

	//-----------------------------------
	// Create the texture from a given (inclusive) address range.
	// Refer to pGlideState->pTexMemory for the texture memory.
	// We can depend on DTDState::CreateAt() to setup our creation info.
	// Given: m_Info, m_iBegin, m_iEnd, nWidth, and nHeight
	virtual bool Create();

	// Announce that the texture needs updating (texture buffer became dirty).
	virtual void Update();

	// Announce that the palette needs updating.
	// (Palette buffer is dirty, and we're about to draw!)
	virtual void UpdatePal();

	// Toasts the texture from memory. (May be called multiple times)
	virtual void Toast();

	//-----------------------------------
	// Determine if the texture is identical to the one given in pInfo.
	virtual bool IsEqual( GrTexInfo *pInfo );

	//-----------------------------------
	// Remember the texture information.
	GrTexInfo m_Info;

	// Index to the begin addres, and to the end address. (Defines an inclusive range)
	DWORD m_iBegin, m_iEnd;

	// For double linked list management.
	VTexture *m_pPrev, *m_pNext;

	// The usual Direct3D containers.
	LPDIRECTDRAWSURFACE4			pSurface;	// Direct3D texture surface.
	LPDIRECT3DTEXTURE2				pTexture;	// Texture querried interface.
	LPDIRECTDRAWPALETTE				pPalette;	// Palette container. (For 8bit textures)

	// Generic parsed texture information.
	bool	bPal;		// Determine if this texture has a palette associated with it.
	int		nWidth;		// Texture width.
	int		nHeight;	// Texture height.
};


#endif // __VTexture_h_

// TBI - Add in support for CVS (version control) in all the source files.

//-------------------------------------------------------------------
//	History:
//	$Log:$
//-------------------------------------------------------------------

