//-------------------------------------------------------------------
//	File:		VertexProcessor.h
//	Created:	23/02/99 3:25AM
//	Author:		Aaron Hilton
//	Comments:	Process a Glide vertex into Direct3D format in the most optimal form.
//  Thanks goes to Scott Cutler for the wonderful vertex
//  processing idea, and most of the code!
//-------------------------------------------------------------------
#ifndef __VertexProcessor_h_
#define __VertexProcessor_h_

#define MAX_VERTS 1024
extern D3DTLVERTEX vertexBuffer[MAX_VERTS];

extern void VertexProcessor( const GrVertex *vert, int nOffset );

// Currently selected vertex processing functions.
extern void (*VP_Snap)(); // Pointer to the vertex snapping funciton. (Updated by DGDraw functions)
extern void (*VP_Depth)(); // Depth buffer vertex processing. (Updated by GlideState::FlushStates)
extern void (*VP_Color)(); // Pointer to the color processing function. (Updated by GlideState::FlushStates)
extern void (*VP_Texture)(); // Pointer to the texture coordinate calculation function. (Updated by GlideState::FlushStates)


void VP_Null(); // Generic NULL function pointer.

void VP_P_Snap(); // Perform vertex snapping.
void VP_P_NoSnap(); // Just copy the vertex coordinate.
void VP_P_Snap_Flip(); // Perform vertex snapping with origin in lower left (flip Y).
void VP_P_NoSnap_Flip(); // Just copy the vertex coordinates with origin in lower left (flip Y).

// Depth buffer vertex processing.
void VP_Depth_WBuffer(); // W-Buffer conversion.  (Scott Cutler's method)
void VP_Depth_ZBuffer(); // Regular Z-Buffer conversion.

// Color processing.
void VP_Color_II(); // Color Iterated, and Alpha Iterated.
void VP_Color_IC(); // Color Iterated, and Alpha Constant.
void VP_Color_CI(); // Color Constant, and Alpha Iterated.
void VP_Color_CC(); // Color Constant, and Alpha Constant.

// Texture coordinate processing.
void VP_Texture_W();
void VP_Texture_No_W();

#endif // __VertexProcessor_h_
//-------------------------------------------------------------------
//	History:
//	$Log:$
//-------------------------------------------------------------------
